@@ -16,23 +16,48 @@
  * Copyright (C) 2003, 2004, 2006 Øyvind Kolås
  */
 
-#include <gtk/gtk.h>
-#include "gegl-plugin.h"  /* FIXME: should just be gegl.h */
+#include "config.h"
+
 #include <string.h>
 #include <stdlib.h>
+
+#include <gtk/gtk.h>
+
+#include "gegl-bin-gui-types.h"
+
+#include "gegl-plugin.h"  /* FIXME: should just be gegl.h */
+
 #include "editor.h"
 #include "gegl-tree-editor.h"
 #include "gegl-node-editor.h"
 #include "gegl-tree-editor-action.h"
 
-extern GeglNode *editor_output;
 
-static void entry_activate (GtkEntry * entry, gpointer user_data);
+static void           popup_properties                       (GeglNode           *node);
+static void           chain_in_operation                     (const gchar        *op_type);
+static void           menu_item_activate                     (GtkWidget          *widget,
+                                                              gpointer            user_data);
+static void           operation_class_iterate_for_completion (GType               type,
+                                                              GtkListStore       *store);
+static GtkTreeModel * create_completion_model                (GeglNode           *item);
+static void           operation_class_iterate_for_menu       (GType               type,
+                                                              GtkWidget          *menu,
+                                                              GeglNode           *item);
+static GtkWidget *    optype_menu                            (GeglNode           *item);
+static void           gtk_option_menu_position               (GtkMenu            *menu,
+                                                              gint               *x,
+                                                              gint               *y,
+                                                              gboolean           *push_in,
+                                                              gpointer            user_data);
+static void           button_clicked                         (GtkButton          *button,
+                                                              gpointer            item);
+static void           entry_activate                         (GtkEntry           *entry,
+                                                              gpointer            user_data);
+static gboolean       completion_match_selected              (GtkEntryCompletion *completion,
+                                                              GtkTreeModel       *model,
+                                                              GtkTreeIter        *iter,
+                                                              gpointer            user_data);
 
-static gboolean
-completion_match_selected (GtkEntryCompletion * completion,
-                           GtkTreeModel *model,
-                           GtkTreeIter *iter, gpointer user_data);
 
 static void popup_properties (GeglNode *node)
 {
@@ -49,10 +74,10 @@ static void popup_properties (GeglNode *node)
   gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), vbox);
 
   g_signal_connect_swapped (dialog,
-                            "response", 
+                            "response",
                             G_CALLBACK (gtk_widget_destroy),
                             dialog);
-  
+
   gtk_widget_show_all (dialog);
 }
 
@@ -84,9 +109,9 @@ chain_in_operation (const gchar *op_type)
   popup_properties (new);
 }
 
-
 static void
-menu_item_activate (GtkWidget *widget, gpointer user_data)
+menu_item_activate (GtkWidget *widget,
+                    gpointer   user_data)
 {
   GtkWidget *menu_item;
   GtkWidget *label;
@@ -122,7 +147,7 @@ menu_item_activate (GtkWidget *widget, gpointer user_data)
     }
 #if 0
 
-  item = user_data;
+  item     = user_data;
   old_type = oxide_get_property (item, "type");
   old_name = oxide_get_property (item, "name");
   oxide_set_property (item, "type", new_type);
@@ -136,14 +161,14 @@ menu_item_activate (GtkWidget *widget, gpointer user_data)
     property_editor_rebuild (bauxite->property_editor, item);
 
   if (old_name)
-    free (old_name);
+   g_free (old_name);
   if (old_type)
-    free (old_type);
+    g_free (old_type);
 #endif
 }
 
 static void
-operation_class_iterate_for_completion (GType type,
+operation_class_iterate_for_completion (GType         type,
                                         GtkListStore *store)
 {
   GType *ops;
@@ -197,7 +222,7 @@ operation_class_iterate_for_menu (GType      type,
     return;
   ops = g_type_children (type, &children);
 
-  for (no=0; no<children; no++)
+  for (no = 0; no <children; no++)
     {
       GeglOperationClass *klass;
 
@@ -391,17 +416,10 @@ button_clicked (GtkButton * button, gpointer item)
   gtk_menu_popup (menu, NULL, NULL, gtk_option_menu_position, button, 0, 0);
 }
 
-/* FIXME: this should be in a header, here to silence gcc in extreme
- * strictness of error reporting mode
- */
 GtkWidget *
-typeeditor_optype (GtkSizeGroup *col1,
-                   GtkSizeGroup *col2,
-                   GeglNodeEditor *node_editor);
-GtkWidget *
-typeeditor_optype (GtkSizeGroup *col1,
-                   GtkSizeGroup *col2,
-                   GeglNodeEditor *node_editor)
+gegl_typeeditor_optype (GtkSizeGroup   *col1,
+                        GtkSizeGroup   *col2,
+                        GeglNodeEditor *node_editor)
 {
   GtkWidget *hbox;
   GtkWidget *label = NULL;
@@ -413,7 +431,7 @@ typeeditor_optype (GtkSizeGroup *col1,
   GtkWidget *button_arrow;
   char     *current_type;
   GeglNode  *item = NULL;
-  
+
   if (node_editor)
     item =node_editor->node;
 
@@ -454,7 +472,7 @@ typeeditor_optype (GtkSizeGroup *col1,
   if (current_type)
     {
       gtk_entry_set_text (GTK_ENTRY (entry), current_type);
-      free (current_type);
+      g_free (current_type);
     }
   gtk_button_set_focus_on_click (GTK_BUTTON (button), FALSE);
   gtk_entry_set_completion (GTK_ENTRY (entry), completion);
@@ -476,7 +494,8 @@ typeeditor_optype (GtkSizeGroup *col1,
 }
 
 static void
-entry_activate (GtkEntry * entry, gpointer user_data)
+entry_activate (GtkEntry *entry,
+                gpointer  user_data)
 {
   GeglNode   *item = user_data;
   const char *new_type = gtk_entry_get_text (entry);
@@ -516,6 +535,6 @@ completion_match_selected (GtkEntryCompletion *completion,
   gtk_entry_set_text (GTK_ENTRY (entry), new_value);
   entry_activate (GTK_ENTRY (entry), user_data);
 
-  free (new_value);
+  g_free (new_value);
   return TRUE;
 }
