@@ -16,19 +16,32 @@
  * Copyright (C) 2003, 2004, 2006 Øyvind Kolås
  */
 
+#include "config.h"
+
+#include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #include <limits.h>
 #include <stdlib.h>
 #include <string.h>
+
+#include "gegl-bin-gui-types.h"
+
 #include "gegl.h"
+
+#include "editor-optype.h"
+#include "gegl-node-editor.h"
+#include "gegl-options.h"
 #include "gegl-store.h"
 #include "gegl-tree-editor.h"
-#include "gegl-node-editor.h"
-#include "editor.h"
 #include "gegl-view.h"
-#include <gdk/gdkkeysyms.h>
+#include "gegl-view.h"
+
+#include "editor.h"
 
 #ifdef G_OS_WIN32
+#ifndef PATH_MAX
+#define PATH_MAX _MAX_PATH
+#endif
 #define realpath(a,b) _fullpath(b,a,_MAX_PATH)
 #endif
 
@@ -60,11 +73,6 @@ cb_window_keybinding (GtkWidget *widget, GdkEventKey *event, gpointer data)
   return FALSE;
 }
 
-GtkWidget *
-typeeditor_optype (GtkSizeGroup   *col1,
-                   GtkSizeGroup   *col2,
-                   GeglNodeEditor *node_editor);
-
 Editor editor;
 
 static void editor_set_gegl (GeglNode    *gegl);
@@ -120,7 +128,7 @@ create_window (Editor *editor)
   gtk_paned_pack2 (GTK_PANED (hpaned_top), editor->tree_editor, FALSE, TRUE);
 
     {
-      GtkWidget *foo = typeeditor_optype (NULL, NULL, NULL);
+      GtkWidget *foo = gegl_typeeditor_optype (NULL, NULL, NULL);
       gtk_box_pack_start (GTK_BOX (add_box), foo, TRUE, TRUE, 0);
     }
 
@@ -156,7 +164,9 @@ GeglNode *editor_output = NULL;
 static void cb_shrinkwrap (GtkAction *action);
 static void cb_fit (GtkAction *action);
 static void cb_fit_on_screen (GtkAction *action);
+#if 0
 static void cb_recompute (GtkAction *action);
+#endif
 static void cb_redraw (GtkAction *action);
 static void cb_next_file (GtkAction *action);
 static void cb_previous_file (GtkAction *action);
@@ -351,10 +361,12 @@ static GtkActionEntry action_entries[] = {
    "",
    G_CALLBACK (cb_zoom_200)},
 
+#if 0
   {"Recompute", NULL,
    "_Recompute View", "<shift><control>R",
    "Recalculate all image data (for working around dirt bugs)",
    G_CALLBACK (cb_recompute)},
+#endif
 
   {"Redraw", NULL,
    "_Redraw View", "<control>R",
@@ -392,12 +404,12 @@ static const gchar *ui_info =
   "      <menuitem action='Zoom200'/>"
   "      <separator/>"
   "      <menuitem action='Redraw'/>"
-  "      <menuitem action='Recompute'/>"
+  /*"      <menuitem action='Recompute'/>"*/
   "      <separator/>"
   "      <menuitem action='Structure'/>"
   "      <menuitem action='Tree'/>"
   "      <menuitem action='Properties'/>"
-  "      <menuitem action='Introspect'/>"
+  /*"      <menuitem action='Introspect'/>"*/
   "    </menu>"
   "    <menu action='HelpMenu'>"
   "      <menuitem action='About'/>"
@@ -950,16 +962,19 @@ static void cb_shrinkwrap (GtkAction *action)
   }*/
 }
 
-void          gegl_node_disable_cache       (GeglNode      *node);
-
+#if 0
 static void cb_recompute (GtkAction *action)
 {
   GeglNode *node;
 
   g_object_get (GEGL_VIEW(editor.view), "node", &node, NULL);
-  gegl_node_disable_cache (node);
+  /*gegl_node_disable_cache (node);*/
+  this used to just forcibly remove the existing cache object for the
+    toplevel cache.
+  */
   gegl_gui_flush ();
 }
+#endif
 
 static void cb_redraw (GtkAction *action)
 {
@@ -1151,6 +1166,10 @@ void editor_refresh_structure (void)
                            GTK_TREE_MODEL (store));
 }
 
+typedef struct _GeglPad GeglPad;
+void gegl_pad_set_format (gpointer,gpointer);
+gpointer gegl_node_get_pad (gpointer, const gchar *name);
+
 static void editor_set_gegl (GeglNode    *gegl)
 {
 /*
@@ -1162,6 +1181,13 @@ static void editor_set_gegl (GeglNode    *gegl)
     g_object_unref (editor.gegl);
   editor.gegl = gegl;
 
+    {
+      GeglPad *pad;
+      pad = gegl_node_get_pad (gegl, "output");
+      g_assert (pad);
+      gegl_pad_set_format (pad, babl_format ("R'G'B' u8"));
+    }
+
   g_object_set (editor.view, "node", editor.gegl, NULL);
   editor_refresh_structure ();
 }
