@@ -17,11 +17,16 @@
  */
 
 #include "config.h"
+
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
+
+#include "gegl-bin-types.h"
+
 #include "gegl-options.h"
 
+
 static GeglOptions *opts_new (void)
 {
   GeglOptions *o = g_malloc0 (sizeof (GeglOptions));
@@ -227,7 +232,9 @@ parse_args (int    argc,
 
         else if (match ("--file") ||
                  match ("-i")) {
-            o->files = g_list_append (o->files, g_strdup (*curr));
+            const gchar *file_path;
+            get_string (file_path);
+            o->files = g_list_append (o->files, g_strdup (file_path));
         }
 
         else if (match ("--xml") ||
