@@ -16,13 +16,21 @@
  * Copyright (C) 2003, 2004, 2006 Øyvind Kolås
  */
 
+#include "config.h"
+
 #include <gtk/gtk.h>
+
+#include "gegl-bin-gui-types.h"
+
+#include "gegl.h"
+
+#include "editor.h"
+#include "gegl-node-editor.h"
 #include "gegl-plugin.h"
+#include "gegl-store.h"
+#include "gegl-tree-editor-action.h"
 #include "gegl-tree-editor.h"
-#include "gegl-node-editor.h"
-#include "editor.h"
 
-extern GeglNode *editor_output;
 
 void property_editor_rebuild (GtkWidget *container,
                               GeglNode  *node)
@@ -237,6 +245,7 @@ tree_editor_new (GtkWidget *property_editor)
 
   gtk_box_pack_start (GTK_BOX (bot_vbox), tree_scroll, TRUE, TRUE, 0);
 
+#if 0
   if(0){
     GtkWidget *butbox = gtk_hbox_new (FALSE, 1);
     GtkWidget *button;
@@ -275,6 +284,7 @@ tree_editor_new (GtkWidget *property_editor)
 
     gtk_box_pack_start (GTK_BOX (bot_vbox), butbox, FALSE, FALSE, 0);
   }
+#endif
 
   self = bot_vbox;
 
