@@ -27,9 +27,9 @@ This is the graph we're going to construct:
    |   .------.
    |   | text |
    |   `------'
-.-----------------.
-| FractalExplorer |
-`-----------------'
+.------------------.
+| fractal-explorer |
+`------------------'
 
 */
 
@@ -46,9 +46,9 @@ This is the graph we're going to construct:
                                  "color", gegl_color_new ("rgb(1.0,1.0,1.0)"),
                                  NULL);
     GeglNode *mandelbrot = gegl_node_new_child (gegl,
-                                "operation", "FractalExplorer",
-                                "width", 256,
-                                "height", 256,
+                                "operation", "fractal-explorer",
+                                "width", 512,
+                                "height", 384,
                                 NULL);
 
     gegl_node_link_many (mandelbrot, layer, display, NULL);
@@ -59,7 +59,7 @@ This is the graph we're going to construct:
      */
     {
       gint frame;
-      gint frames = 30;
+      gint frames = 200;
 
       for (frame=0; frame<frames; frame++)
         {
