@@ -16,13 +16,25 @@
  * Copyright 2006, 2007 Øyvind Kolås <pippin@gimp.org>
  */
 
-#include <fcntl.h>
-#include <unistd.h>
+#include "config.h"
+
 #include <string.h>
-#include <glib.h>
+#include <errno.h>
+
+#include <fcntl.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
+#include <glib-object.h>
 #include <glib/gstdio.h>
 
-#include "../gegl-types.h"
+#ifdef G_OS_WIN32
+#include <io.h>
+#endif
+
+#include "gegl-types.h"
+
 #include "gegl-buffer-types.h"
 #include "gegl-buffer.h"
 #include "gegl-storage.h"
@@ -52,7 +64,8 @@ typedef struct
 } LoadInfo;
 
 
-static void load_info_destroy (LoadInfo *info)
+static void
+load_info_destroy (LoadInfo *info)
 {
   if (!info)
     return;
@@ -66,20 +79,19 @@ static void load_info_destroy (LoadInfo *info)
       GList *iter;
       for (iter = info->tiles; iter; iter = iter->next)
         {
-          GeglTileEntry *entry = iter->data;
-          g_free (entry);
+          g_slice_free (GeglTileEntry, iter->data);
         }
       g_list_free (info->tiles);
       info->tiles = NULL;
     }
-  g_free (info);
+  g_slice_free (LoadInfo, info);
 }
 
 void
 gegl_buffer_load (GeglBuffer  *buffer,
                   const gchar *path)
 {
-  LoadInfo *info = g_malloc0 (sizeof (LoadInfo));
+  LoadInfo *info = g_slice_new0 (LoadInfo);
 
   if (sizeof (GeglBufferFileHeader) != 256)
     {
@@ -96,7 +108,12 @@ gegl_buffer_load (GeglBuffer  *buffer,
   info->fd   = g_open (info->path, O_RDONLY, 0);
   if (info->fd == -1)
     {
-      g_message ("Unable to open '%s' for loading a buffer", info->path);
+      gchar *name = g_filename_display_name (info->path);
+
+      g_message ("Unable to open '%s' for loading a buffer: %s",
+                 name, g_strerror (errno));
+      g_free (name);
+
       load_info_destroy (info);
       return;
     }
@@ -113,12 +130,13 @@ gegl_buffer_load (GeglBuffer  *buffer,
     gint i;
     for (i = 0; i < info->header.tile_count; i++)
       {
-        GeglTile *entry = g_malloc0 (sizeof (GeglTileEntry));
+        GeglTileEntry *entry = g_slice_new0 (GeglTileEntry);
 
         read (info->fd, entry, sizeof (GeglTileEntry));
 
-        info->tiles = g_list_append (info->tiles, entry);
+        info->tiles = g_list_prepend (info->tiles, entry);
       }
+    info->tiles = g_list_reverse (info->tiles);
   }
 
   /* load each tile */
@@ -166,4 +184,3 @@ gegl_buffer_load (GeglBuffer  *buffer,
 
   load_info_destroy (info);
 }
-
