@@ -16,13 +16,31 @@
  * Copyright 2006, 2007 Øyvind Kolås <pippin@gimp.org>
  */
 
-#include <fcntl.h>
-#include <unistd.h>
+#include "config.h"
+
 #include <string.h>
-#include <glib.h>
+#include <errno.h>
+
+#include <fcntl.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
+#include <glib-object.h>
 #include <glib/gstdio.h>
 
-#include "../gegl-types.h"
+#ifdef G_OS_WIN32
+#include <io.h>
+#ifndef S_IRUSR
+#define S_IRUSR _S_IREAD
+#endif
+#ifndef S_IWUSR
+#define S_IWUSR _S_IWRITE
+#endif
+#endif
+
+#include "gegl-types.h"
+
 #include "gegl-buffer-types.h"
 #include "gegl-buffer.h"
 #include "gegl-storage.h"
@@ -50,11 +68,12 @@ typedef struct
 } SaveInfo;
 
 
-GeglTileEntry *tile_entry_new (gint x,
-                               gint y,
-                               gint z)
+static GeglTileEntry *
+tile_entry_new (gint x,
+                gint y,
+                gint z)
 {
-  GeglTileEntry *entry = g_malloc0 (sizeof (GeglTileEntry));
+  GeglTileEntry *entry = g_slice_new0 (GeglTileEntry);
 
   entry->x = x;
   entry->y = y;
@@ -62,12 +81,14 @@ GeglTileEntry *tile_entry_new (gint x,
   return entry;
 }
 
-void tile_entry_destroy (GeglTileEntry *entry)
+static void
+tile_entry_destroy (GeglTileEntry *entry)
 {
-  g_free (entry);
+  g_slice_free (GeglTileEntry, entry);
 }
 
-void save_info_destroy (SaveInfo *info)
+static void
+save_info_destroy (SaveInfo *info)
 {
   if (!info)
     return;
@@ -80,14 +101,11 @@ void save_info_destroy (SaveInfo *info)
     {
       GList *iter;
       for (iter = info->tiles; iter; iter = iter->next)
-        {
-          GeglTileEntry *entry = iter->data;
-          tile_entry_destroy (entry);
-        }
+        tile_entry_destroy (iter->data);
       g_list_free (info->tiles);
       info->tiles = NULL;
     }
-  g_free (info);
+  g_slice_free (SaveInfo, info);
 }
 
 
@@ -125,18 +143,15 @@ static gint z_order_compare (gconstpointer a,
   const GeglTileEntry *entryA = a;
   const GeglTileEntry *entryB = b;
 
-  glong                zA = z_order (entryA);
-  glong                zB = z_order (entryB);
-
-  return zB - zA;
+  return z_order (entryB) - z_order (entryA);
 }
 
 void
-gegl_buffer_save (GeglBuffer    *buffer,
-                  const gchar   *path,
-                  GeglRectangle *roi)
+gegl_buffer_save (GeglBuffer          *buffer,
+                  const gchar         *path,
+                  const GeglRectangle *roi)
 {
-  SaveInfo *info = g_malloc0 (sizeof (SaveInfo));
+  SaveInfo *info = g_slice_new0 (SaveInfo);
 
   if (sizeof (GeglBufferFileHeader) != 256)
     {
@@ -146,10 +161,16 @@ gegl_buffer_save (GeglBuffer    *buffer,
 
   strcpy (info->header.magic, "_G_E_G_L");
   info->path = g_strdup (path);
-  info->fd   = g_open (info->path, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
+  info->fd   = g_open (info->path,
+                       O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
   if (info->fd == -1)
     {
-      g_message ("Unable to open '%s' when saving a buffer", info->path);
+      gchar *name = g_filename_display_name (info->path);
+
+      g_message ("Unable to open '%s' when saving a buffer: %s",
+                 name, g_strerror (errno));
+      g_free (name);
+
       save_info_destroy (info);
       return;
     }
@@ -210,16 +231,15 @@ gegl_buffer_save (GeglBuffer    *buffer,
                   gint tx = gegl_tile_indice (tiledx / factor, tile_width);
                   gint ty = gegl_tile_indice (tiledy / factor, tile_height);
 
-                  if (1)
+                  if (gegl_provider_message (GEGL_PROVIDER (buffer),
+                                             GEGL_TILE_EXIST, tx, ty, z, NULL))
                     {
-                      if (gegl_provider_message (GEGL_PROVIDER (buffer),
-                                                   GEGL_TILE_EXIST, tx, ty, z, NULL))
-                        {
-                          tx += info->x_tile_shift / factor;
-                          ty += info->y_tile_shift / factor;
+                      tx += info->x_tile_shift / factor;
+                      ty += info->y_tile_shift / factor;
 
-                          info->tiles = g_list_append (info->tiles, tile_entry_new (tx, ty, z));
-                        }
+                      info->tiles = g_list_prepend (info->tiles,
+                                                    tile_entry_new (tx, ty, z));
+                      info->header.tile_count++;
                     }
                   bufx += (tile_width - offsetx) * factor;
                 }
@@ -228,9 +248,10 @@ gegl_buffer_save (GeglBuffer    *buffer,
           factor *= 2;
         }
     }
+
+    info->tiles = g_list_reverse (info->tiles);
   }
 
-  info->header.tile_count = g_list_length (info->tiles);
   /* FIXME: sort the index into Z-order */
 
   info->tiles = g_list_sort (info->tiles, z_order_compare);
