@@ -15,14 +15,14 @@
  *
  * Copyright 2006 Øyvind Kolås <pippin@gimp.org>
  */
-#include <glib.h>
+
+#ifndef __GEGL_BUFFER_SAVE_H__
+#define __GEGL_BUFFER_SAVE_H__
+
 #include <glib-object.h>
 
 #include "gegl-buffer.h"
 
-#ifndef _GEGL_BUFFER_SAVE_H
-#define _GEGL_BUFFER_SAVE_H
-
 typedef struct {
   gchar magic[16];
   gint  width, height, x, y;
@@ -43,8 +43,8 @@ typedef struct {
   guint flags;
 } GeglTileEntry;
 
-void gegl_buffer_save (GeglBuffer    *buffer,
-                       const gchar   *path,
-                       GeglRectangle *roi);
+void gegl_buffer_save (GeglBuffer          *buffer,
+                       const gchar         *path,
+                       const GeglRectangle *roi);
 
 #endif
