@@ -16,12 +16,18 @@
  * Copyright 2006,2007 Øyvind Kolås <pippin@gimp.org>
  */
 
-#include <glib.h>
-#include <glib/gstdio.h>
+#include "config.h"
+
+#include <math.h>
+#include <string.h>
+
+#include <glib-object.h>
+
+#include "gegl-types.h"
 
-#include "../gegl-types.h"
 #include "gegl-buffer-types.h"
 #include "gegl-buffer.h"
+#include "gegl-buffer-allocator.h"
 #include "gegl-storage.h"
 #include "gegl-tile-backend.h"
 #include "gegl-handler.h"
@@ -30,9 +36,13 @@
 #include "gegl-handler-log.h"
 #include "gegl-handler-empty.h"
 #include "gegl-buffer-allocator.h"
+#include "gegl-sampler-nearest.h"
+#include "gegl-sampler-linear.h"
+#include "gegl-sampler-cubic.h"
 #include "gegl-types.h"
 #include "gegl-utils.h"
 
+
 G_DEFINE_TYPE (GeglBuffer, gegl_buffer, GEGL_TYPE_TILE_TRAITS)
 
 static GObjectClass * parent_class = NULL;
@@ -68,7 +78,7 @@ static inline gint needed_width (gint w,
   return needed_tiles (w, stride) * stride;
 }
 
-static void *int_gegl_buffer_get_format (GeglBuffer *buffer);
+static const void *int_gegl_buffer_get_format (GeglBuffer *buffer);
 static void
 get_property (GObject    *gobject,
               guint       property_id,
@@ -104,7 +114,7 @@ get_property (GObject    *gobject,
         if (buffer->format == NULL)
           buffer->format = int_gegl_buffer_get_format (buffer);
 
-        g_value_set_pointer (value, buffer->format);
+        g_value_set_pointer (value, (void*)buffer->format); /* Eeeek? */
         break;
 
       case PROP_X:
@@ -208,20 +218,13 @@ gint gegl_buffer_leaks (void)
   return allocated_buffers - de_allocated_buffers;
 }
 
-#include "gegl-buffer-allocator.h"
-
-#include <string.h>
-
 static void gegl_buffer_void (GeglBuffer *buffer);
 
 static void
 gegl_buffer_dispose (GObject *object)
 {
-  GeglBuffer    *buffer;
-  GeglHandler *handler;
-
-  buffer = (GeglBuffer *) object;
-  handler  = GEGL_HANDLER (object);
+  GeglBuffer  *buffer  = GEGL_BUFFER (object);
+  GeglHandler *handler = GEGL_HANDLER (object);
 
   gegl_buffer_sample_cleanup (buffer);
 
@@ -242,7 +245,8 @@ gegl_buffer_dispose (GObject *object)
 
   de_allocated_buffers++; /* XXX: is it correct to count that, shouldn't that
                              only be counted in finalize? */
-  (*G_OBJECT_CLASS (parent_class)->dispose)(object);
+
+  G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static GeglTileBackend *
@@ -385,7 +389,7 @@ gegl_buffer_constructor (GType                  type,
   else if (buffer->abyss.width == -1 ||
            buffer->abyss.height == -1)
     {
-      buffer->abyss.x      = GEGL_BUFFER (provider)->abyss.x - buffer->shift_x; 
+      buffer->abyss.x      = GEGL_BUFFER (provider)->abyss.x - buffer->shift_x;
       buffer->abyss.y      = GEGL_BUFFER (provider)->abyss.y - buffer->shift_y;
       buffer->abyss.width  = GEGL_BUFFER (provider)->abyss.width;
       buffer->abyss.height = GEGL_BUFFER (provider)->abyss.height;
@@ -501,16 +505,11 @@ get_tile (GeglProvider *tile_store,
   return tile;
 }
 
-#include "limits.h"
-
 static void
 gegl_buffer_class_init (GeglBufferClass *class)
 {
-  GObjectClass       *gobject_class;
-  GeglProviderClass *tile_provider_class;
-
-  gobject_class    = (GObjectClass *) class;
-  tile_provider_class = (GeglProviderClass *) class;
+  GObjectClass      *gobject_class       = G_OBJECT_CLASS (class);
+  GeglProviderClass *tile_provider_class = GEGL_PROVIDER_CLASS (class);
 
   parent_class                = g_type_class_peek_parent (class);
   gobject_class->dispose      = gegl_buffer_dispose;
@@ -586,10 +585,11 @@ gegl_buffer_class_init (GeglBufferClass *class)
 static void
 gegl_buffer_init (GeglBuffer *buffer)
 {
-  buffer->extent.x            = 0;
-  buffer->extent.y            = 0;
-  buffer->extent.width        = 0;
-  buffer->extent.height       = 0;
+  buffer->extent.x      = 0;
+  buffer->extent.y      = 0;
+  buffer->extent.width  = 0;
+  buffer->extent.height = 0;
+
   buffer->shift_x      = 0;
   buffer->shift_y      = 0;
   buffer->abyss.x      = 0;
@@ -616,6 +616,8 @@ gegl_buffer_void_tile (GeglBuffer *buffer,
 {
   gint z = 0;
 
+  g_return_val_if_fail (GEGL_IS_BUFFER (buffer), NULL);
+
   return gegl_provider_message (GEGL_PROVIDER (buffer),
                                   GEGL_TILE_VOID, x, y, z, NULL);
 }
@@ -624,14 +626,16 @@ gegl_buffer_void_tile (GeglBuffer *buffer,
 gboolean
 gegl_buffer_idle (GeglBuffer *buffer)
 {
+  g_return_val_if_fail (GEGL_IS_BUFFER (buffer), FALSE);
+
   return gegl_provider_message (GEGL_PROVIDER (buffer),
-                                  GEGL_TILE_IDLE, 0, 0, 0, NULL);
+                                GEGL_TILE_IDLE, 0, 0, 0, NULL);
 }
 #endif
 
 /***************************************************************************/
 
-static void *int_gegl_buffer_get_format (GeglBuffer *buffer)
+static const void *int_gegl_buffer_get_format (GeglBuffer *buffer)
 {
   g_assert (buffer);
   if (buffer->format != NULL)
@@ -699,8 +703,6 @@ done_with_row:
  * scanline basis instead of allocating large temporary buffers. (using babl for "memcpy")
  */
 
-#include <string.h>
-
 #ifdef BABL
 #undef BABL
 #endif
@@ -717,7 +719,7 @@ static inline void
 pset (GeglBuffer *buffer,
       gint        x,
       gint        y,
-      Babl       *format,
+      const Babl *format,
       guchar     *buf)
 {
   gint  tile_width  = buffer->storage->tile_width;
@@ -783,7 +785,7 @@ static inline void
 pset (GeglBuffer *buffer,
       gint        x,
       gint        y,
-      Babl       *format,
+      const Babl *format,
       gpointer    data)
 {
   guchar *buf         = data;
@@ -802,7 +804,8 @@ pset (GeglBuffer *buffer,
 
   if (format != buffer->format)
     {
-      fish = babl_fish (buffer->format, format);
+      fish = babl_fish ((gpointer) buffer->format,
+                        (gpointer) format);
     }
 
   {
@@ -866,7 +869,7 @@ static inline void
 pget (GeglBuffer *buffer,
       gint        x,
       gint        y,
-      Babl       *format,
+      const Babl *format,
       gpointer    data)
 {
   guchar *buf         = data;
@@ -885,7 +888,8 @@ pget (GeglBuffer *buffer,
 
   if (format != buffer->format)
     {
-      fish = babl_fish (buffer->format, format);
+      fish = babl_fish ((gpointer) buffer->format,
+                        (gpointer) format);
     }
 
   {
@@ -948,6 +952,8 @@ pget (GeglBuffer *buffer,
 void
 gegl_buffer_flush (GeglBuffer *buffer)
 {
+  g_return_if_fail (GEGL_IS_BUFFER (buffer));
+
   if (buffer->hot_tile)
     {
       g_object_unref (buffer->hot_tile);
@@ -961,7 +967,7 @@ gegl_buffer_iterate (GeglBuffer *buffer,
                      guchar     *buf,
                      gint        rowstride,
                      gboolean    write,
-                     Babl       *format,
+                     const Babl *format,
                      gint        level)
 {
   gint  width       = buffer->extent.width;
@@ -1012,11 +1018,13 @@ gegl_buffer_iterate (GeglBuffer *buffer,
     {
       if (write)
         {
-          fish = babl_fish (format, buffer->format);
+          fish = babl_fish ((gpointer) format,
+                            (gpointer) buffer->format);
         }
       else
         {
-          fish = babl_fish (buffer->format, format);
+          fish = babl_fish ((gpointer) buffer->format,
+                            (gpointer) format);
         }
     }
 
@@ -1025,7 +1033,7 @@ gegl_buffer_iterate (GeglBuffer *buffer,
       gint tiledy  = buffer_y + bufy;
       gint offsety = gegl_tile_offset (tiledy, tile_height);
 
-      
+
 
       gint bufx    = 0;
 
@@ -1133,7 +1141,7 @@ gegl_buffer_iterate (GeglBuffer *buffer,
                             if (buffer_y + y >= buffer_abyss_y &&
                                 buffer_y + y < abyss_y_total)
                               {
-                                babl_process (fish, bp + lskip * bpx_size, tp + lskip * px_size, 
+                                babl_process (fish, bp + lskip * bpx_size, tp + lskip * px_size,
                                  pixels - lskip - rskip);
                               }
 
@@ -1212,11 +1220,14 @@ gegl_buffer_iterate (GeglBuffer *buffer,
 void
 gegl_buffer_set (GeglBuffer          *buffer,
                  const GeglRectangle *rect,
-                 Babl                *format,
-                 void                *src)
+                 const Babl          *format,
+                 void                *src,
+                 gint                 rowstride)
 {
   GeglBuffer *sub_buf;
 
+  g_return_if_fail (GEGL_IS_BUFFER (buffer));
+
   if (format == NULL)
     format = buffer->format;
 
@@ -1234,11 +1245,11 @@ gegl_buffer_set (GeglBuffer          *buffer,
    */
   if (rect == NULL)
     {
-      gegl_buffer_iterate (buffer, src, GEGL_AUTO_ROWSTRIDE, TRUE, format, 0);
+      gegl_buffer_iterate (buffer, src, rowstride, TRUE, format, 0);
       return;
     }
   sub_buf = gegl_buffer_create_sub_buffer (buffer, rect);
-  gegl_buffer_iterate (sub_buf, src, GEGL_AUTO_ROWSTRIDE, TRUE, format, 0);
+  gegl_buffer_iterate (sub_buf, src, rowstride, TRUE, format, 0);
   g_object_unref (sub_buf);
 }
 
@@ -1250,12 +1261,13 @@ gegl_buffer_set (GeglBuffer          *buffer,
  * level:  halving levels 0 = 1:1 1=1:2 2=1:4 3=1:8 ..
  *
  */
-static void gegl_buffer_get_scaled (GeglBuffer          *buffer,
-                                    const GeglRectangle *rect,
-                                    void                *dst,
-                                    gint                 rowstride,
-                                    void                *format,
-                                    gint                 level)
+static void
+gegl_buffer_get_scaled (GeglBuffer          *buffer,
+                        const GeglRectangle *rect,
+                        void                *dst,
+                        gint                 rowstride,
+                        const void          *format,
+                        gint                 level)
 {
   GeglBuffer *sub_buf = gegl_buffer_create_sub_buffer (buffer, rect);
   gegl_buffer_iterate (sub_buf, dst, rowstride, FALSE, format, level);
@@ -1269,17 +1281,18 @@ static void gegl_buffer_get_scaled (GeglBuffer          *buffer,
  *  completely correct.
  */
 
-static void resample_nearest (void   *dest_buf,
-                              void   *source_buf,
-                              gint    dest_w,
-                              gint    dest_h,
-                              gint    source_w,
-                              gint    source_h,
-                              gdouble offset_x,
-                              gdouble offset_y,
-                              gdouble scale,
-                              gint    bpp,
-                              gint    rowstride)
+static void
+resample_nearest (void   *dest_buf,
+                  void   *source_buf,
+                  gint    dest_w,
+                  gint    dest_h,
+                  gint    source_w,
+                  gint    source_h,
+                  gdouble offset_x,
+                  gdouble offset_y,
+                  gdouble scale,
+                  gint    bpp,
+                  gint    rowstride)
 {
   gint x, y;
 
@@ -1318,35 +1331,40 @@ static void resample_nearest (void   *dest_buf,
 }
 #endif
 
-/* Optimized|obfuscated version of the nearest neighbour resampler 
+/* Optimized|obfuscated version of the nearest neighbour resampler
  * XXX: seems to contains some very slight inprecision in the rendering.
  */
-static void resample_nearest (void   *dest_buf,
-                              void   *source_buf,
-                              gint    dest_w,
-                              gint    dest_h,
-                              gint    source_w,
-                              gint    source_h,
-                              gdouble offset_x,
-                              gdouble offset_y,
-                              gdouble scale,
-                              gint    bpp,
-                              gint    rowstride)
+static void
+resample_nearest (void   *dest_buf,
+                  void   *source_buf,
+                  gint    dest_w,
+                  gint    dest_h,
+                  gint    source_w,
+                  gint    source_h,
+                  gdouble offset_x,
+                  gdouble offset_y,
+                  gdouble scale,
+                  gint    bpp,
+                  gint    rowstride)
 {
   gint x, y;
+  guint xdiff, ydiff, xstart, sy;
 
   if (rowstride == GEGL_AUTO_ROWSTRIDE)
      rowstride = dest_w * bpp;
 
-  guint xdiff = 65536 / scale;
-  guint ydiff = 65536 / scale;
-  guint xstart = (offset_x * 65536) / scale;
-  guint sy = (offset_y * 65536) / scale;
+  xdiff = 65536 / scale;
+  ydiff = 65536 / scale;
+  xstart = (offset_x * 65536) / scale;
+  sy = (offset_y * 65536) / scale;
 
   for (y = 0; y < dest_h; y++)
     {
       guchar *dst;
       guchar *src_base;
+      guint sx;
+      guint px = 0;
+      guchar *src;
 
       if (sy >= source_h << 16)
         sy = (source_h - 1) << 16;
@@ -1354,11 +1372,8 @@ static void resample_nearest (void   *dest_buf,
       dst      = ((guchar *) dest_buf) + y * rowstride;
       src_base = ((guchar *) source_buf) + (sy >> 16) * source_w * bpp;
 
-      guint sx;
-      guint px = 0; 
-
       sx = xstart;
-      guchar *src = src_base;
+      src = src_base;
 
       /* this is the loop that is actually properly optimized,
        * portions of the setup is done for all the rows outside the y
@@ -1411,17 +1426,18 @@ box_filter (guint          left_weight,
     }
 }
 
-static void resample_boxfilter_u8 (void   *dest_buf,
-                                   void   *source_buf,
-                                   gint    dest_w,
-                                   gint    dest_h,
-                                   gint    source_w,
-                                   gint    source_h,
-                                   gdouble offset_x,
-                                   gdouble offset_y,
-                                   gdouble scale,
-                                   gint    components,
-                                   gint    rowstride)
+static void
+resample_boxfilter_u8 (void   *dest_buf,
+                       void   *source_buf,
+                       gint    dest_w,
+                       gint    dest_h,
+                       gint    source_w,
+                       gint    source_h,
+                       gdouble offset_x,
+                       gdouble offset_y,
+                       gdouble scale,
+                       gint    components,
+                       gint    rowstride)
 {
   gint x, y;
   gint iscale      = scale * 256;
@@ -1440,8 +1456,8 @@ static void resample_boxfilter_u8 (void   *dest_buf,
   guint         middle_weight;
   guint         bottom_weight;
 
-  footprint_y = (1.0 / scale) * 256; 
-  footprint_x = (1.0 / scale) * 256; 
+  footprint_y = (1.0 / scale) * 256;
+  footprint_x = (1.0 / scale) * 256;
   foosum = footprint_x * footprint_y;
 
   if (rowstride != GEGL_AUTO_ROWSTRIDE)
@@ -1453,6 +1469,8 @@ static void resample_boxfilter_u8 (void   *dest_buf,
       gint    dy;
       guchar *dst;
       const guchar *src_base;
+      gint sx;
+      gint xdelta;
 
       sy = ((y + offset_y) * 65536) / iscale;
 
@@ -1476,8 +1494,8 @@ static void resample_boxfilter_u8 (void   *dest_buf,
 
       middle_weight = footprint_y - top_weight - bottom_weight;
 
-      gint sx = (offset_x *65536) / iscale;
-      gint xdelta = 65536/iscale;
+      sx = (offset_x *65536) / iscale;
+      xdelta = 65536/iscale;
 
       /* XXX: needs quite a bit of optimization */
       for (x = 0; x < dest_w; x++)
@@ -1556,16 +1574,15 @@ static void resample_boxfilter_u8 (void   *dest_buf,
     }
 }
 
-#include <math.h>
-
 void
 gegl_buffer_get (GeglBuffer          *buffer,
                  gdouble              scale,
                  const GeglRectangle *rect,
-                 Babl                *format,
+                 const Babl          *format,
                  gpointer             dest_buf,
                  gint                 rowstride)
 {
+  g_return_if_fail (GEGL_IS_BUFFER (buffer));
 
   if (format == NULL)
     format = buffer->format;
@@ -1631,7 +1648,7 @@ gegl_buffer_get (GeglBuffer          *buffer,
       sample_buf = g_malloc (buf_width * buf_height * bpp);
       gegl_buffer_get_scaled (buffer, &sample_rect, sample_buf, GEGL_AUTO_ROWSTRIDE, format, level);
 
-      if (BABL (format)->format.type[0] == (BablType *) babl_type ("u8") 
+      if (BABL (format)->format.type[0] == (BablType *) babl_type ("u8")
           && !(level == 0 && scale > 1.99))
         { /* do box-filter resampling if we're 8bit (which projections are) */
 
@@ -1671,22 +1688,22 @@ gegl_buffer_get (GeglBuffer          *buffer,
 const GeglRectangle *
 gegl_buffer_get_abyss (GeglBuffer *buffer)
 {
+  g_return_val_if_fail (GEGL_IS_BUFFER (buffer), NULL);
+
   return &buffer->abyss;
 }
 
-#include "gegl-sampler-nearest.h"
-#include "gegl-sampler-linear.h"
-#include "gegl-sampler-cubic.h"
-
 void
 gegl_buffer_sample (GeglBuffer       *buffer,
                     gdouble           x,
                     gdouble           y,
                     gdouble           scale,
                     gpointer          dest,
-                    Babl             *format,
+                    const Babl       *format,
                     GeglInterpolation interpolation)
 {
+  g_return_if_fail (GEGL_IS_BUFFER (buffer));
+
 /*#define USE_WORKING_SHORTCUT*/
 #ifdef USE_WORKING_SHORTCUT
   pget (buffer, x, y, format, dest);
@@ -1731,6 +1748,8 @@ gegl_buffer_sample (GeglBuffer       *buffer,
 void
 gegl_buffer_sample_cleanup (GeglBuffer *buffer)
 {
+  g_return_if_fail (GEGL_IS_BUFFER (buffer));
+
   if (buffer->sampler)
     {
       g_object_unref (buffer->sampler);
@@ -1741,12 +1760,14 @@ gegl_buffer_sample_cleanup (GeglBuffer *buffer)
 const GeglRectangle *
 gegl_buffer_get_extent (GeglBuffer *buffer)
 {
+  g_return_val_if_fail (GEGL_IS_BUFFER (buffer), NULL);
+
   return &(buffer->extent);
 }
 
 GeglBuffer *
 gegl_buffer_new (const GeglRectangle *extent,
-                 Babl                *format)
+                 const Babl          *format)
 {
   GeglRectangle empty={0,0,0,0};
 
@@ -1765,10 +1786,13 @@ gegl_buffer_new (const GeglRectangle *extent,
                        NULL);
 }
 
-GeglBuffer* 
+GeglBuffer*
 gegl_buffer_create_sub_buffer (GeglBuffer          *buffer,
                                const GeglRectangle *extent)
 {
+  g_return_val_if_fail (GEGL_IS_BUFFER (buffer), NULL);
+  g_return_val_if_fail (extent != NULL, NULL);
+
   return g_object_new (GEGL_TYPE_BUFFER,
                        "provider", buffer,
                        "x", extent->x,
@@ -1790,13 +1814,14 @@ gegl_buffer_copy (GeglBuffer          *src,
 
   GeglRectangle src_line;
   GeglRectangle dst_line;
-  gpointer      format;
+  const Babl   *format;
   guchar       *temp;
   guint         i;
   gint          pxsize;
 
-  g_assert (src);
-  g_assert (dst);
+  g_return_if_fail (GEGL_IS_BUFFER (src));
+  g_return_if_fail (GEGL_IS_BUFFER (dst));
+
   if (!src_rect)
     {
       src_rect = gegl_buffer_get_extent (src);
@@ -1822,7 +1847,7 @@ gegl_buffer_copy (GeglBuffer          *src,
   for (i=0; i<src_rect->height; i++)
     {
       gegl_buffer_get (src, 1.0, &src_line, format, temp, GEGL_AUTO_ROWSTRIDE);
-      gegl_buffer_set (dst, &dst_line, format, temp);
+      gegl_buffer_set (dst, &dst_line, format, temp, GEGL_AUTO_ROWSTRIDE);
       src_line.y++;
       dst_line.y++;
     }
@@ -1834,7 +1859,8 @@ gegl_buffer_dup (GeglBuffer *buffer)
 {
   GeglBuffer *new;
 
-  g_assert (buffer);
+  g_return_val_if_fail (GEGL_IS_BUFFER (buffer), NULL);
+
   new = gegl_buffer_new (gegl_buffer_get_extent (buffer), buffer->format);
   gegl_buffer_copy (buffer, gegl_buffer_get_extent (buffer),
                     new, gegl_buffer_get_extent (buffer));
@@ -1845,6 +1871,8 @@ gegl_buffer_dup (GeglBuffer *buffer)
 void
 gegl_buffer_destroy (GeglBuffer *buffer)
 {
+  g_return_if_fail (GEGL_IS_BUFFER (buffer));
+
   g_object_unref (buffer);
 }
 
@@ -1852,12 +1880,12 @@ GeglInterpolation
 gegl_buffer_interpolation_from_string (const gchar *string)
 {
   if (g_str_equal (string, "nearest") ||
-      g_str_equal (string, "none")
-   )
+      g_str_equal (string, "none"))
     return GEGL_INTERPOLATION_NEAREST;
+
   if (g_str_equal (string, "linear") ||
-      g_str_equal (string, "bilinear")
-   )
+      g_str_equal (string, "bilinear"))
     return GEGL_INTERPOLATION_LINEAR;
+
  return GEGL_INTERPOLATION_NEAREST;
 }
