@@ -23,7 +23,7 @@
 #include "gegl-provider.h"
 #include "gegl-tile-backend.h"
 
-G_DEFINE_TYPE (GeglTileBackend, gegl_tile_backend, GEGL_TYPE_TILE_STORE)
+G_DEFINE_TYPE (GeglTileBackend, gegl_tile_backend, GEGL_TYPE_PROVIDER)
 static GObjectClass * parent_class = NULL;
 
 enum
@@ -63,6 +63,7 @@ get_property (GObject    *gobject,
         break;
 
       case PROP_FORMAT:
+        g_value_set_pointer (value, backend->format);
         break;
 
       default:
@@ -110,12 +111,10 @@ constructor (GType                  type,
   object  = G_OBJECT_CLASS (parent_class)->constructor (type, n_params, params);
   backend = GEGL_TILE_BACKEND (object);
 
-  g_assert (backend->tile_width > 0 &&
-            backend->tile_height > 0);
-
+  g_assert (backend->tile_width > 0 && backend->tile_height > 0);
   g_assert (backend->format);
-  backend->px_size = ((Babl *) (backend->format))->format.bytes_per_pixel;
 
+  backend->px_size = backend->format->format.bytes_per_pixel;
   backend->tile_size = backend->tile_width * backend->tile_height * backend->px_size;
 
   return object;
@@ -125,10 +124,9 @@ constructor (GType                  type,
 static void
 gegl_tile_backend_class_init (GeglTileBackendClass *klass)
 {
-  GObjectClass *gobject_class;
+  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
   parent_class  = g_type_class_peek_parent (klass);
-  gobject_class = G_OBJECT_CLASS (klass);
 
   gobject_class->set_property = set_property;
   gobject_class->get_property = get_property;
@@ -137,11 +135,13 @@ gegl_tile_backend_class_init (GeglTileBackendClass *klass)
   g_object_class_install_property (gobject_class, PROP_TILE_WIDTH,
                                    g_param_spec_int ("tile-width", "tile-width", "Tile width in pixels",
                                                      0, G_MAXINT, 0,
-                                                     G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+                                                     G_PARAM_READWRITE |
+                                                     G_PARAM_CONSTRUCT_ONLY));
   g_object_class_install_property (gobject_class, PROP_TILE_HEIGHT,
                                    g_param_spec_int ("tile-height", "tile-height", "Tile height in pixels",
                                                      0, G_MAXINT, 0,
-                                                     G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+                                                     G_PARAM_READWRITE |
+                                                     G_PARAM_CONSTRUCT_ONLY));
   g_object_class_install_property (gobject_class, PROP_TILE_SIZE,
                                    g_param_spec_int ("tile-size", "tile-size", "Size of the tiles linear buffer in bytes",
                                                      0, G_MAXINT, 0,
@@ -153,7 +153,7 @@ gegl_tile_backend_class_init (GeglTileBackendClass *klass)
   g_object_class_install_property (gobject_class, PROP_FORMAT,
                                    g_param_spec_pointer ("format", "format", "babl format",
                                                          G_PARAM_READWRITE |
-                                                         G_PARAM_CONSTRUCT));
+                                                         G_PARAM_CONSTRUCT_ONLY));
 }
 
 static void
