@@ -15,12 +15,10 @@
  *
  * Copyright 2006, 2007 Øyvind Kolås <pippin@gimp.org>
  */
-#ifndef _TILE_BACKEND_H
-#define _TILE_BACKEND_H
 
-#include <glib.h>
-#include "gegl-buffer-types.h"
-#include "gegl-tile.h"
+#ifndef __GEGL_TILE_BACKEND_H__
+#define __GEGL_TILE_BACKEND_H__
+
 #include "gegl-provider.h"
 
 G_BEGIN_DECLS
@@ -35,11 +33,12 @@ G_BEGIN_DECLS
 struct _GeglTileBackend
 {
   GeglProvider  parent_instance;
-  gint           tile_width;
-  gint           tile_height;
-  void          *format;        /* defaults to the babl format "R'G'B'A u8" */
-  gint           px_size;       /* size of a single pixel in bytes */
-  gint           tile_size;     /* size of an entire tile in bytes */
+
+  gint          tile_width;
+  gint          tile_height;
+  Babl         *format;        /* defaults to the babl format "R'G'B'A u8" */
+  gint          px_size;       /* size of a single pixel in bytes */
+  gint          tile_size;     /* size of an entire tile in bytes */
 };
 
 struct _GeglTileBackendClass
