@@ -22,29 +22,13 @@
 
 G_BEGIN_DECLS
 
-typedef struct _GeglConnection       GeglConnection;
+#ifndef __GEGL_H__
 typedef struct _GeglColor            GeglColor;
 typedef struct _GeglCurve            GeglCurve;
-typedef struct _GeglCRVisitor        GeglCRVisitor;
-typedef struct _GeglDebugRectVisitor GeglDebugRectVisitor;
-typedef struct _GeglEvalMgr          GeglEvalMgr;
-typedef struct _GeglEvalVisitor      GeglEvalVisitor;
-typedef struct _GeglFinishVisitor    GeglFinishVisitor;
-typedef struct _GeglGraph            GeglGraph;
-typedef struct _GeglHaveVisitor      GeglHaveVisitor;
-typedef struct _GeglNeedVisitor      GeglNeedVisitor;
 typedef struct _GeglNode             GeglNode;
-typedef struct _GeglNodeDynamic      GeglNodeDynamic;
-typedef struct _GeglOperation        GeglOperation;
-typedef struct _GeglPad              GeglPad;
+typedef struct _GeglVector           GeglVector;
 typedef struct _GeglProcessor        GeglProcessor;
-typedef struct _GeglPrepareVisitor   GeglPrepareVisitor;
-typedef struct _GeglVisitable        GeglVisitable; /* dummy typedef */
-typedef struct _GeglVisitor          GeglVisitor;
-
 typedef struct _GeglRectangle        GeglRectangle;
-typedef struct _GeglPoint            GeglPoint;
-typedef struct _GeglDimension        GeglDimension;
 
 struct _GeglRectangle
 {
@@ -54,6 +38,34 @@ struct _GeglRectangle
   gint height;
 };
 
+typedef enum
+{
+  GEGL_PARAM_PAD_OUTPUT = 1 << G_PARAM_USER_SHIFT,
+  GEGL_PARAM_PAD_INPUT  = 1 << (G_PARAM_USER_SHIFT + 1)
+} GeglPadType;
+
+#endif
+typedef struct _GeglCRVisitor        GeglCRVisitor;
+typedef struct _GeglDebugRectVisitor GeglDebugRectVisitor;
+typedef struct _GeglEvalMgr          GeglEvalMgr;
+typedef struct _GeglEvalVisitor      GeglEvalVisitor;
+typedef struct _GeglFinishVisitor    GeglFinishVisitor;
+typedef struct _GeglGraph            GeglGraph;
+typedef struct _GeglHaveVisitor      GeglHaveVisitor;
+typedef struct _GeglNeedVisitor      GeglNeedVisitor;
+#ifndef __GEGL_PLUGIN_H__
+typedef struct _GeglPad              GeglPad;
+typedef struct _GeglOperation        GeglOperation;
+typedef struct _GeglNodeContext      GeglNodeContext;
+typedef struct _GeglConnection       GeglConnection;
+#endif
+typedef struct _GeglPrepareVisitor   GeglPrepareVisitor;
+typedef struct _GeglVisitable        GeglVisitable; /* dummy typedef */
+typedef struct _GeglVisitor          GeglVisitor;
+
+typedef struct _GeglPoint            GeglPoint;
+typedef struct _GeglDimension        GeglDimension;
+
 struct _GeglPoint
 {
   gint x;
@@ -66,6 +78,7 @@ struct _GeglDimension
   gint height;
 };
 
+
 G_END_DECLS
 
 #endif /* __GEGL_TYPES_H__ */
