@@ -26,24 +26,15 @@
 #include "gegl-types.h"
 
 
-void gegl_log         (GLogLevelFlags  level,
-                       gchar          *file,
-                       gint            line,
-                       gchar          *function,
-                       gchar          *format,
-                       ...);
-void gegl_logv        (GLogLevelFlags  level,
-                       gchar          *file,
-                       gint            line,
-                       gchar          *function,
-                       gchar          *format,
-                       va_list         args);
-void gegl_direct_log  (GLogLevelFlags  level,
-                       gchar          *format,
-                       ...);
-void gegl_direct_logv (GLogLevelFlags  level,
-                       gchar          *format,
-                       va_list         args);
+static void gegl_logv        (GLogLevelFlags  level,
+                              const gchar    *file,
+                              gint            line,
+                              const gchar    *function,
+                              const gchar    *format,
+                              va_list         args);
+static void gegl_direct_logv (GLogLevelFlags  level,
+                              const gchar    *format,
+                              va_list         args);
 
 
 inline gint
@@ -52,7 +43,7 @@ _gegl_float_epsilon_zero (float value)
   return value > -GEGL_FLOAT_EPSILON && value < GEGL_FLOAT_EPSILON;
 }
 
-inline gint
+gint
 _gegl_float_epsilon_equal (float v1, float v2)
 {
   register float diff = v1 - v2;
@@ -187,11 +178,12 @@ gegl_rectangle_equal_coords (const GeglRectangle *r,
 
 #define GEGL_LOG_DOMAIN    "Gegl"
 
-void gegl_log_debug (gchar *file,
-                     gint   line,
-                     gchar *function,
-                     gchar *format,
-                     ...)
+void
+gegl_log_debug (const gchar *file,
+                gint         line,
+                const gchar *function,
+                const gchar *format,
+                ...)
 {
   va_list args;
 
@@ -200,11 +192,12 @@ void gegl_log_debug (gchar *file,
   va_end (args);
 }
 
-void gegl_log_info (gchar *file,
-                    gint   line,
-                    gchar *function,
-                    gchar *format,
-                    ...)
+void
+gegl_log_info (const gchar *file,
+               gint         line,
+               const gchar *function,
+               const gchar *format,
+               ...)
 {
   va_list args;
 
@@ -213,11 +206,12 @@ void gegl_log_info (gchar *file,
   va_end (args);
 }
 
-void gegl_log_message (gchar *file,
-                       gint   line,
-                       gchar *function,
-                       gchar *format,
-                       ...)
+void
+gegl_log_message (const gchar *file,
+                  gint         line,
+                  const gchar *function,
+                  const gchar *format,
+                  ...)
 {
   va_list args;
 
@@ -226,8 +220,9 @@ void gegl_log_message (gchar *file,
   va_end (args);
 }
 
-void gegl_log_direct (gchar *format,
-                      ...)
+void
+gegl_log_direct (const gchar *format,
+                 ...)
 {
   va_list args;
 
@@ -236,28 +231,13 @@ void gegl_log_direct (gchar *format,
   va_end (args);
 }
 
-void
-gegl_log (GLogLevelFlags level,
-          gchar         *file,
-          gint           line,
-          gchar         *function,
-          gchar         *format,
-          ...)
-{
-  va_list args;
-
-  va_start (args, format);
-  gegl_logv (level, file, line, function, format, args);
-  va_end (args);
-}
-
-void
-gegl_logv (GLogLevelFlags level,
-           gchar         *file,
-           gint           line,
-           gchar         *function,
-           gchar         *format,
-           va_list        args)
+static void
+gegl_logv (GLogLevelFlags  level,
+           const gchar    *file,
+           gint            line,
+           const gchar    *function,
+           const gchar    *format,
+           va_list         args)
 {
   if (g_getenv ("GEGL_LOG_ON"))
     {
@@ -274,22 +254,10 @@ gegl_logv (GLogLevelFlags level,
     }
 }
 
-void
-gegl_direct_log (GLogLevelFlags level,
-                 gchar         *format,
-                 ...)
-{
-  va_list args;
-
-  va_start (args, format);
-  gegl_direct_logv (level, format, args);
-  va_end (args);
-}
-
-void
-gegl_direct_logv (GLogLevelFlags level,
-                  gchar         *format,
-                  va_list        args)
+static void
+gegl_direct_logv (GLogLevelFlags  level,
+                  const gchar    *format,
+                  va_list         args)
 {
   if (g_getenv ("GEGL_LOG_ON"))
     {
