@@ -13,7 +13,7 @@
  * You should have received a copy of the GNU Lesser General Public
  * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
  *
- * 2000-2007 © Calvin Williamson, Øyvind Kolås.
+ * 2000-2008 © Calvin Williamson, Øyvind Kolås.
  */
 
 #ifndef __GEGL_H__
@@ -107,6 +107,31 @@ GOptionGroup * gegl_get_option_group     (void);
 void           gegl_exit                 (void);
 
 
+/***
+ * GEGL version information
+ *
+ * These macros tell the version of GEGL you are compiling against.
+ * GEGL's version number consists of three parts: major, minor and
+ * micro.
+ */
+
+#define GEGL_MAJOR_VERSION 0
+#define GEGL_MINOR_VERSION 0
+#define GEGL_MICRO_VERSION 16
+
+/**
+ * gegl_get_version:
+ * @major: a pointer to a int where the major version number will be stored
+ * @minor: ditto for the minor version number
+ * @micro: ditto for the micro version number
+ *
+ * This function fetches the version of the GEGL library being used by
+ * the running process.
+ */
+void           gegl_get_version          (int *major,
+                                          int *minor,
+                                          int *micro);
+
 /***
  * Available operations:
  * Gegl provides means to check for available processing operations that
@@ -177,14 +202,18 @@ GParamSpec** gegl_list_properties           (const gchar   *operation_type,
  *                                  NULL);
  */
 #ifndef GEGL_INTERNAL /* These declarations duplicate internal ones in GEGL */
-typedef struct _GeglNode      GeglNode;
+
+#ifndef GEGL_TYPE_NODE
+typedef struct _GeglNode  GeglNode;
 GType gegl_node_get_type  (void) G_GNUC_CONST;
-#define GEGL_TYPE_NODE  (gegl_node_get_type())
-#define GEGL_NODE(obj)  (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEGL_TYPE_NODE, GeglNode))
+#define GEGL_TYPE_NODE    (gegl_node_get_type())
+#define GEGL_NODE(obj)    (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEGL_TYPE_NODE, GeglNode))
+#define GEGL_IS_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEGL_TYPE_NODE))
+#endif
 
 typedef struct _GeglRectangle GeglRectangle;
-GType gegl_rectangle_get_type  (void) G_GNUC_CONST;
-#define GEGL_TYPE_RECTANGLE  (gegl_rectangle_get_type())
+GType gegl_rectangle_get_type (void) G_GNUC_CONST;
+#define GEGL_TYPE_RECTANGLE   (gegl_rectangle_get_type())
 #endif
 
 /**
@@ -216,7 +245,7 @@ GeglNode     * gegl_node_new             (void);
  * Returns a newly created node. The node will be destroyed by the parent.
  * Calling g_object_unref on a node will cause the node to be dropped by the
  * parent. (You may also add additional references using
- * g_object_ref/g_objecr_unref, but in general relying on the parents reference
+ * g_object_ref/g_object_unref, but in general relying on the parents reference
  * counting is easiest.)
  */
 GeglNode    * gegl_node_new_child        (GeglNode      *parent,
@@ -232,7 +261,7 @@ GeglNode    * gegl_node_new_child        (GeglNode      *parent,
  * ---
  * gegl_node_link_many (background, over, png_save, NULL);
  * gegl_node_connect_to (translate, "output", over, "aux");
- * gegl_node_link_many (text, blur, translate);
+ * gegl_node_link_many (text, blur, translate, NULL);
  */
 
 /**
@@ -312,9 +341,9 @@ gboolean      gegl_node_disconnect       (GeglNode      *node,
  * #gegl_node_new_child as well as later when changing the initial
  * value with #gegl_node_set.
  *
- * To see what operations are available for a given operation look in the <a
+ * To see what properties are available for a given operation look in the <a
  * href='operations.html'>Operations reference</a> or use
- * #gegl_node_get_properties.
+ * #gegl_node_get.
  */
 
 /**
@@ -369,7 +398,8 @@ void          gegl_node_get              (GeglNode      *node,
  * sample.
  */
 
-#ifndef GEGL_INTERNAL
+#ifndef GEGL_BLIT_FLAGS
+#define  GEGL_BLIT_FLAGS
 typedef enum
 {
   GEGL_BLIT_DEFAULT  = 0,
@@ -402,13 +432,13 @@ typedef enum
  *
  * Render a rectangular region from a node.
  */
-void          gegl_node_blit             (GeglNode      *node,
-                                          gdouble        scale,
-                                          GeglRectangle *roi,
-                                          Babl          *format,
-                                          gpointer       destination_buf,
-                                          gint           rowstride,
-                                          GeglBlitFlags  flags);
+void          gegl_node_blit             (GeglNode            *node,
+                                          gdouble              scale,
+                                          const GeglRectangle *roi,
+                                          const Babl          *format,
+                                          gpointer             destination_buf,
+                                          gint                 rowstride,
+                                          GeglBlitFlags        flags);
 
 /**
  * gegl_node_process:
@@ -450,6 +480,40 @@ void          gegl_node_blit             (GeglNode      *node,
 void          gegl_node_process          (GeglNode      *sink_node);
 
 
+/***
+ * Reparenting:
+ *
+ * Sometimes it is useful to be able to move nodes between graphs or even
+ * handle orphaned nods that are not part of a graph. gegl_node_adopt_child
+ * and gegl_node_get_parent are provided ot handle such cases.
+ */
+
+/**
+ * gegl_node_adopt_child:
+ * @parent: a #GeglNode or NULL.
+ * @child: a #GeglNode
+ *
+ * Adds @child to the responsibilities of @node, this makes the parent node
+ * take a reference on the child that is kept as long as the parent itself is
+ * being referenced. The node is stolen from an existing parent if there is one,
+ * or a presumed existing reference is used. If @parent is NULL the child will
+ * be orphaned and the developer is given a reference to be responsible of.
+ *
+ * Returns the child, or NULL if there was a problem with the arguments.
+ */
+GeglNode    * gegl_node_adopt_child        (GeglNode      *parent,
+                                            GeglNode      *child);
+
+/**
+ * gegl_node_get_parent:
+ * @node: a #GeglNode
+ *
+ * Returns a GeglNode that keeps a reference on a child.
+ *
+ * Returns the parent of a node or NULL.
+ */
+GeglNode    * gegl_node_get_parent         (GeglNode      *node);
+
 
 /***
  * State queries:
@@ -552,7 +616,7 @@ GeglNode    * gegl_node_get_input_proxy  (GeglNode      *node,
  * NULL if there is no op associated. The special name "GraphNode"
  * is returned if the node is the container of a subgraph.
  */
-const gchar * gegl_node_get_operation    (GeglNode      *node);
+const gchar * gegl_node_get_operation    (const GeglNode *node);
 
 /**
  * gegl_node_get_output_proxy:
@@ -697,10 +761,11 @@ gchar       * gegl_node_to_xml           (GeglNode      *node,
  *
  */
 #ifndef GEGL_INTERNAL
+typedef struct _GeglProcessor  GeglProcessor;
 GType gegl_processor_get_type  (void) G_GNUC_CONST;
-typedef struct _GeglProcessor      GeglProcessor;
-#define GEGL_TYPE_PROCESSOR  (gegl_processor_get_type())
-#define GEGL_PROCESSOR(obj)  (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEGL_TYPE_PROCESSOR, GeglProcessor))
+#define GEGL_TYPE_PROCESSOR    (gegl_processor_get_type())
+#define GEGL_PROCESSOR(obj)    (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEGL_TYPE_PROCESSOR, GeglProcessor))
+#define GEGL_IS_PROCESSOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEGL_TYPE_PROCESSOR))
 #endif
 
 /**
@@ -711,8 +776,8 @@ typedef struct _GeglProcessor      GeglProcessor;
  *
  * Returns a new #GeglProcessor.
  */
-GeglProcessor *gegl_node_new_processor      (GeglNode      *node,
-                                             GeglRectangle *rectangle);
+GeglProcessor *gegl_node_new_processor      (GeglNode            *node,
+                                             const GeglRectangle *rectangle);
 
 /**
  * gegl_processor_set_rectangle:
@@ -722,8 +787,8 @@ GeglProcessor *gegl_node_new_processor      (GeglNode      *node,
  *
  * Change the rectangle a #GeglProcessor is working on.
  */
-void           gegl_processor_set_rectangle (GeglProcessor *processor,
-                                             GeglRectangle *rectangle);
+void           gegl_processor_set_rectangle (GeglProcessor       *processor,
+                                             const GeglRectangle *rectangle);
 
 
 /**
@@ -757,67 +822,6 @@ void           gegl_processor_destroy       (GeglProcessor *processor);
 
 
 
-#ifndef GEGL_INTERNAL
-
-
-/***
- * GeglColor:
- *
- * GeglColor is used for properties that use a gegl color, use #gegl_color_new
- * with a NULL string to create a new blank one, gegl_colors are destroyed
- * with g_object_unref when they no longer are needed.
- *
- * The colors used by gegls are described in a format similar to CSS. The
- * textstring "rgb(1.0,1.0,1.0)" signifies opaque white and
- * "rgba(1.0,0.0,0.0,0.75)" is a 75% opaque red. Hexadecimal forms like #RRGGBB
- * and #RRGGBBAA are also supported.
- */
-typedef struct _GeglColor     GeglColor;
-GType gegl_color_get_type (void) G_GNUC_CONST;
-#define GEGL_TYPE_COLOR (gegl_color_get_type())
-#define GEGL_COLOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEGL_TYPE_COLOR, GeglColor))
-#endif
-
-/**
- * gegl_color_new:
- * @string: a CSS-style color string.
- *
- * Returns a #GeglColor object suitable for use with #gegl_node_set.
- */
-GeglColor   * gegl_color_new                (const gchar   *string);
-
-/**
- * gegl_color_get_rgba:
- * @color: a #GeglColor
- * @r: return location for red component.
- * @g: return location for green component.
- * @b: return location for blue component.
- * @a: return location for alpha component.
- *
- * Retrieve RGB component values from a #GeglColor.
- */
-void          gegl_color_get_rgba           (GeglColor     *color,
-                                             gfloat        *r,
-                                             gfloat        *g,
-                                             gfloat        *b,
-                                             gfloat        *a);
-
-/**
- * gegl_color_set_rgba:
- * @color: a #GeglColor
- * @r: new value for red component
- * @g: new value for green component
- * @b: new value for blue component
- * @a: new value for alpha component
- *
- * Retrieve RGB component values from a GeglColor.
- */
-void          gegl_color_set_rgba           (GeglColor     *color,
-                                             gfloat         r,
-                                             gfloat         g,
-                                             gfloat         b,
-                                             gfloat         a);
-
 #ifndef GEGL_INTERNAL
 /***
  * GeglRectangle:
@@ -842,61 +846,25 @@ struct _GeglRectangle
   gint height;
 };
 
-/***
- * GeglCurve:
- *
- * Documentation and function signatures left out of documentation until
- * API is frozen.
- */
-typedef struct _GeglCurve       GeglCurve;
-GType        gegl_curve_get_type            (void) G_GNUC_CONST;
-#define GEGL_TYPE_CURVE            (gegl_curve_get_type ())
-#define GEGL_CURVE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEGL_TYPE_CURVE, GeglCurve))
+#define GEGL_PAD_TYPE
+
+typedef enum
+{
+  GEGL_PARAM_PAD_OUTPUT = 1 << G_PARAM_USER_SHIFT,
+  GEGL_PARAM_PAD_INPUT  = 1 << (G_PARAM_USER_SHIFT + 1)
+} GeglPadType;
+
+typedef struct _GeglCurve  GeglCurve;
+typedef struct _GeglVector  GeglVector;
+typedef struct _GeglColor  GeglColor;
 #endif
 
-GeglCurve*   gegl_curve_new                 (gdouble         y_min,
-                                             gdouble         y_max);
 
-GeglCurve*   gegl_curve_default_curve       (void) G_GNUC_CONST;
-
-void         gegl_curve_get_y_bounds        (GeglCurve      *self,
-                                             gdouble        *min_y,
-                                             gdouble        *max_y);
-
-guint        gegl_curve_add_point           (GeglCurve      *self,
-                                             gdouble         x,
-                                             gdouble         y);
-
-void         gegl_curve_remove_point_index  (GeglCurve      *self,
-                                             guint           index);
-
-void         gegl_curve_get_point           (GeglCurve      *self,
-                                             guint           index,
-                                             gdouble        *x,
-                                             gdouble        *y);
-
-void         gegl_curve_set_point           (GeglCurve      *self,
-                                             guint           index,
-                                             gdouble         x,
-                                             gdouble         y);
- 
-guint        gegl_curve_num_points          (GeglCurve      *self);
-
-gdouble     gegl_curve_calc_value           (GeglCurve      *self,
-                                             gdouble         x);
-
-void         gegl_curve_calc_values         (GeglCurve      *self,
-                                             gdouble         x_min,
-                                             gdouble         x_max,
-                                             guint           num_samples,
-                                             gdouble        *xs,
-                                             gdouble        *ys);
+/*** foo
+ */
+#include <gegl-buffer.h>
+#include <gegl-color.h>
+#include <gegl-curve.h>
 
 G_END_DECLS
-
-/*** this is just here to trick the parser.
- */
-#include "gegl/property-types/gegl-paramspecs.h"
-#include <babl/babl.h>
-
 #endif  /* __GEGL_H__ */
