@@ -17,13 +17,15 @@
  *           2006 Øyvind Kolås
  */
 
+#define GEGL_INTERNAL
+
 #include "config.h"
 
-#include <stdio.h>
 #include <string.h>
 
 #include <glib-object.h>
 #include <gobject/gvaluecollector.h>
+
 #include "gegl-types.h"
 
 #include "gegl-node.h"
@@ -31,15 +33,17 @@
 #include "gegl-pad.h"
 #include "gegl-utils.h"
 #include "gegl-visitable.h"
+
 #include "operation/gegl-operation.h"
+#include "operation/gegl-operations.h"
 #include "operation/gegl-operation-meta.h"
+
 #include "process/gegl-eval-mgr.h"
 #include "process/gegl-have-visitor.h"
 #include "process/gegl-prepare-visitor.h"
 #include "process/gegl-finish-visitor.h"
 #include "process/gegl-processor.h"
 
-
 enum
 {
   PROP_0,
@@ -48,19 +52,28 @@ enum
   PROP_NAME
 };
 
-typedef struct _GeglNodePriv
+enum
+{
+  INVALIDATED,
+  COMPUTED,
+  LAST_SIGNAL
+};
+
+
+struct _GeglNodePrivate
 {
   GSList         *children;  /*  used for children */
   GeglNode       *parent;
   gchar          *name;
-} GeglNodePriv;
+  GeglProcessor  *processor;
+};
 
 
 #define GEGL_NODE_GET_PRIVATE(obj) \
-  ((GeglNodePriv *)(((GeglNode *) obj)->priv))
+  ((GeglNodePrivate *)(((GeglNode *) obj)->priv))
 
 
-guint gegl_node_signals[GEGL_NODE_LAST_SIGNAL] = {0};
+static guint gegl_node_signals[LAST_SIGNAL] = {0};
 
 
 static void            gegl_node_class_init           (GeglNodeClass *klass);
@@ -109,7 +122,7 @@ gegl_node_class_init (GeglNodeClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
-  g_type_class_add_private (klass, sizeof (GeglNodePriv));
+  g_type_class_add_private (klass, sizeof (GeglNodePrivate));
 
   gobject_class->finalize     = finalize;
   gobject_class->dispose      = dispose;
@@ -117,12 +130,13 @@ gegl_node_class_init (GeglNodeClass *klass)
   gobject_class->get_property = get_property;
 
   g_object_class_install_property (gobject_class, PROP_OPERATION,
-                                   g_param_spec_object ("gegl_operation",
+                                   g_param_spec_object ("gegl-operation",
                                                         "Operation Object",
                                                         "The associated GeglOperation instance",
                                                         GEGL_TYPE_OPERATION,
-                                                        G_PARAM_WRITABLE |
+                                                        G_PARAM_READWRITE |
                                                         G_PARAM_CONSTRUCT));
+
   g_object_class_install_property (gobject_class, PROP_OP_CLASS,
                                    g_param_spec_string ("operation",
                                                         "Operation Type",
@@ -139,37 +153,35 @@ gegl_node_class_init (GeglNodeClass *klass)
                                                         G_PARAM_CONSTRUCT |
                                                         G_PARAM_READWRITE));
 
-  gegl_node_signals[GEGL_NODE_INVALIDATED] =
-    g_signal_new ("invalidated", G_TYPE_FROM_CLASS (klass),
+  gegl_node_signals[INVALIDATED] =
+    g_signal_new ("invalidated",
+                  G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
-                  0 /* class offset*/,
-                  NULL /* accumulator */,
-                  NULL /* accu_data */,
+                  0,
+                  NULL, NULL,
                   g_cclosure_marshal_VOID__BOXED,
-                  G_TYPE_NONE /* return type */,
-                  1 /* n_params */,
-                  GEGL_TYPE_RECTANGLE /* param_types */);
+                  G_TYPE_NONE, 1,
+                  GEGL_TYPE_RECTANGLE);
 
-  gegl_node_signals[GEGL_NODE_COMPUTED] =
-    g_signal_new ("computed", G_TYPE_FROM_CLASS (klass),
+  gegl_node_signals[COMPUTED] =
+    g_signal_new ("computed",
+                  G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
-                  0 /* class offset*/,
-                  NULL /* accumulator */,
-                  NULL /* accu_data */,
+                  0,
+                  NULL, NULL,
                   g_cclosure_marshal_VOID__BOXED,
-                  G_TYPE_NONE /* return type */,
-                  1 /* n_params */,
-                  GEGL_TYPE_RECTANGLE /* param_types */);
+                  G_TYPE_NONE, 1,
+                  GEGL_TYPE_RECTANGLE);
 }
 
 static void
 gegl_node_init (GeglNode *self)
 {
-  GeglNodePriv *priv;
+  GeglNodePrivate *priv;
 
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
                                             GEGL_TYPE_NODE,
-                                            GeglNodePriv);
+                                            GeglNodePrivate);
 
   priv = GEGL_NODE_GET_PRIVATE (self);
 
@@ -186,6 +198,7 @@ gegl_node_init (GeglNode *self)
   priv->parent      = NULL;
   priv->children    = NULL;
   priv->name        = NULL;
+  priv->processor   = NULL;
 }
 
 static void
@@ -202,8 +215,8 @@ visitable_init (gpointer ginterface,
 static void
 dispose (GObject *gobject)
 {
-  GeglNode     *self = GEGL_NODE (gobject);
-  GeglNodePriv *priv = GEGL_NODE_GET_PRIVATE (self);
+  GeglNode        *self = GEGL_NODE (gobject);
+  GeglNodePrivate *priv = GEGL_NODE_GET_PRIVATE (self);
 
   if (priv->parent != NULL)
     {
@@ -213,16 +226,25 @@ dispose (GObject *gobject)
     }
 
   gegl_node_remove_children (self);
-  gegl_node_disable_cache (self);
+  if (self->cache)
+    {
+      g_object_unref (self->cache);
+      self->cache = NULL;
+    }
 
+  if (priv->processor)
+    {
+      gegl_processor_destroy (priv->processor);
+      priv->processor = NULL;
+    }
   G_OBJECT_CLASS (gegl_node_parent_class)->dispose (gobject);
 }
 
 static void
 finalize (GObject *gobject)
 {
-  GeglNode     *self = GEGL_NODE (gobject);
-  GeglNodePriv *priv = GEGL_NODE_GET_PRIVATE (self);
+  GeglNode        *self = GEGL_NODE (gobject);
+  GeglNodePrivate *priv = GEGL_NODE_GET_PRIVATE (self);
 
   gegl_node_disconnect_sources (self);
   gegl_node_disconnect_sinks (self);
@@ -258,6 +280,7 @@ set_property (GObject      *gobject,
               GParamSpec   *pspec)
 {
   GeglNode *node = GEGL_NODE (gobject);
+
   switch (property_id)
     {
       case PROP_NAME:
@@ -265,8 +288,7 @@ set_property (GObject      *gobject,
         break;
 
       case PROP_OP_CLASS:
-        gegl_node_set_op_class (node, g_value_get_string (value),
-                                NULL, NULL);
+        gegl_node_set_op_class (node, g_value_get_string (value), NULL, NULL);
         break;
 
       case PROP_OPERATION:
@@ -298,6 +320,10 @@ get_property (GObject    *gobject,
         g_value_set_string (value, gegl_node_get_name (node));
         break;
 
+      case PROP_OPERATION:
+        g_value_set_object (value, node->operation);
+        break;
+
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
         break;
@@ -319,7 +345,9 @@ gegl_node_get_pad (GeglNode    *self,
 {
   GSList *list;
 
-  g_assert (self);
+  g_return_val_if_fail (GEGL_IS_NODE (self), NULL);
+  g_return_val_if_fail (name != NULL, NULL);
+
   if (!self->pads)
     return NULL;
 
@@ -417,8 +445,8 @@ pads_exist (GeglNode    *sink,
       sink_pad = gegl_node_get_pad (sink, sink_pad_name);
       if (!sink_pad || !gegl_pad_is_input (sink_pad))
         {
-          g_warning ("Can't find sink property %s of %s", sink_pad_name,
-                     gegl_node_get_debug_name (sink));
+          g_warning ("%s: Can't find sink property %s of %s", G_STRFUNC,
+                     sink_pad_name, gegl_node_get_debug_name (sink));
           return FALSE;
         }
     }
@@ -429,8 +457,8 @@ pads_exist (GeglNode    *sink,
       source_pad = gegl_node_get_pad (source, source_pad_name);
       if (!source_pad || !gegl_pad_is_output (source_pad))
         {
-          g_warning ("Can't find source property %s of %s", source_pad_name,
-                     gegl_node_get_debug_name (source));
+          g_warning ("%s: Can't find source property %s of %s", G_STRFUNC,
+                     source_pad_name, gegl_node_get_debug_name (source));
           return FALSE;
         }
     }
@@ -457,54 +485,58 @@ find_connection (GeglNode *sink,
   return NULL;
 }
 
-#include <stdio.h>
-
 gboolean
 gegl_node_connect_to (GeglNode    *source,
                       const gchar *source_pad_name,
                       GeglNode    *sink,
                       const gchar *sink_pad_name)
 {
+  g_return_val_if_fail (GEGL_IS_NODE (source), FALSE);
+  g_return_val_if_fail (source_pad_name != NULL, FALSE);
+  g_return_val_if_fail (GEGL_IS_NODE (sink), FALSE);
+  g_return_val_if_fail (sink_pad_name != NULL, FALSE);
+
   return gegl_node_connect_from (sink, sink_pad_name, source, source_pad_name);
 }
 
 void
-gegl_node_invalidated (GeglNode      *node,
-                       GeglRectangle *rect)
+gegl_node_invalidated (GeglNode            *node,
+                       const GeglRectangle *rect)
 {
+  g_return_if_fail (GEGL_IS_NODE (node));
+  g_return_if_fail (rect != NULL);
+
   if (node->cache)
     {
       gegl_cache_invalidate (node->cache, rect);
     }
-  g_signal_emit (node, gegl_node_signals[GEGL_NODE_INVALIDATED],
-                 0, rect, NULL);
+
+  g_signal_emit (node, gegl_node_signals[INVALIDATED], 0,
+                 rect, NULL);
 }
 
 static void
-source_invalidated (GeglNode      *source,
-                    GeglRectangle *rect,
-                    gpointer       data)
+source_invalidated (GeglNode            *source,
+                    const GeglRectangle *rect,
+                    gpointer             data)
 {
-  GeglRectangle dirty_rect;
-  GeglPad      *destination_pad = GEGL_PAD (data);
-  GeglNode     *destination     = GEGL_NODE (gegl_pad_get_node (destination_pad));
-  gchar        *source_name;
-  gchar        *destination_name;
-
-  destination_name = g_strdup (gegl_node_get_debug_name (destination));
-  source_name      = g_strdup (gegl_node_get_debug_name (source));
+  GeglPad       *destination_pad = GEGL_PAD (data);
+  GeglNode      *destination     = gegl_pad_get_node (destination_pad);
+  GeglRectangle  dirty_rect;
 
   if (0) g_warning ("%s.%s is dirtied from %s (%i,%i %ix%i)",
-                    destination_name,
+                    gegl_node_get_debug_name (destination),
                     gegl_pad_get_name (destination_pad),
-                    source_name,
+                    gegl_node_get_debug_name (source),
                     rect->x, rect->y,
                     rect->width, rect->height);
 
   if (destination->operation)
     {
-      dirty_rect = gegl_operation_compute_affected_region (destination->operation,
-                                                       gegl_pad_get_name (destination_pad), *rect);
+      dirty_rect =
+        gegl_operation_get_invalidated_by_change (destination->operation,
+                                                  gegl_pad_get_name (destination_pad),
+                                                  rect);
     }
   else
     {
@@ -512,9 +544,6 @@ source_invalidated (GeglNode      *source,
     }
 
   gegl_node_invalidated (destination, &dirty_rect);
-
-  g_free (source_name);
-  g_free (destination_name);
 }
 
 gboolean
@@ -524,7 +553,9 @@ gegl_node_connect_from (GeglNode    *sink,
                         const gchar *source_pad_name)
 {
   g_return_val_if_fail (GEGL_IS_NODE (sink), FALSE);
+  g_return_val_if_fail (sink_pad_name != NULL, FALSE);
   g_return_val_if_fail (GEGL_IS_NODE (source), FALSE);
+  g_return_val_if_fail (source_pad_name != NULL, FALSE);
 
   {
     GeglPad *pad;
@@ -535,7 +566,8 @@ gegl_node_connect_from (GeglNode    *sink,
       other_pad = gegl_pad_get_connected_to (pad);
     else
       {
-        g_warning ("Didn't find pad '%s' of '%s'", sink_pad_name, gegl_node_get_debug_name (sink));
+        g_warning ("%s: Didn't find pad '%s' of '%s'",
+                   G_STRFUNC, sink_pad_name, gegl_node_get_debug_name (sink));
       }
 
     if (other_pad)
@@ -556,7 +588,8 @@ gegl_node_connect_from (GeglNode    *sink,
       sink->sources = g_slist_prepend (sink->sources, connection);
       source->sinks = g_slist_prepend (source->sinks, connection);
 
-      g_signal_connect (G_OBJECT (source), "invalidated", G_CALLBACK (source_invalidated), sink_pad);
+      g_signal_connect (G_OBJECT (source), "invalidated",
+                        G_CALLBACK (source_invalidated), sink_pad);
 
       property_changed (G_OBJECT (source->operation), NULL, source);
 
@@ -571,6 +604,7 @@ gegl_node_disconnect (GeglNode    *sink,
                       const gchar *sink_pad_name)
 {
   g_return_val_if_fail (GEGL_IS_NODE (sink), FALSE);
+  g_return_val_if_fail (sink_pad_name != NULL, FALSE);
 
   if (pads_exist (sink, sink_pad_name, NULL, NULL))
     {
@@ -590,7 +624,7 @@ gegl_node_disconnect (GeglNode    *sink,
         gulong handler;
 
         handler = g_signal_handler_find (source, G_SIGNAL_MATCH_DATA,
-                                         gegl_node_signals[GEGL_NODE_INVALIDATED],
+                                         gegl_node_signals[INVALIDATED],
                                          0, NULL, NULL, sink_pad);
         if (handler)
           {
@@ -603,7 +637,7 @@ gegl_node_disconnect (GeglNode    *sink,
       sink->sources = g_slist_remove (sink->sources, connection);
       source->sinks = g_slist_remove (source->sinks, connection);
 
-      g_free (connection);
+      gegl_connection_destroy (connection);
 
       return TRUE;
     }
@@ -683,26 +717,33 @@ gegl_node_get_num_sinks (GeglNode *self)
 GSList *
 gegl_node_get_sinks (GeglNode *self)
 {
+  g_return_val_if_fail (GEGL_IS_NODE (self), FALSE);
+
   return self->sinks;
 }
 
-void          gegl_node_link (GeglNode *source,
-                              GeglNode *sink)
+void
+gegl_node_link (GeglNode *source,
+                GeglNode *sink)
 {
+  g_return_if_fail (GEGL_IS_NODE (source));
+  g_return_if_fail (GEGL_IS_NODE (sink));
+
   /* using connect_to is more natural here, but leads to an extra
    * function call, perhaps connect_to and connect_from should be swapped?
    */
   gegl_node_connect_to (source, "output", sink, "input");
 }
 
-void          gegl_node_link_many (GeglNode *source,
-                                   GeglNode *dest,
-                                   ...)
+void
+gegl_node_link_many (GeglNode *source,
+                     GeglNode *dest,
+                     ...)
 {
   va_list var_args;
 
-  g_assert (GEGL_IS_NODE (source));
-  g_assert (GEGL_IS_NODE (dest));
+  g_return_if_fail (GEGL_IS_NODE (source));
+  g_return_if_fail (GEGL_IS_NODE (dest));
 
   va_start (var_args, dest);
   while (dest)
@@ -715,49 +756,57 @@ void          gegl_node_link_many (GeglNode *source,
 }
 
 static GeglBuffer *
-gegl_node_apply_roi (GeglNode      *self,
-                     const gchar   *output_pad_name,
-                     GeglRectangle *roi)
+gegl_node_apply_roi (GeglNode            *self,
+                     const gchar         *output_pad_name,
+                     const GeglRectangle *roi)
 {
   GeglEvalMgr *eval_mgr;
   GeglBuffer  *buffer;
 
-  g_assert (GEGL_IS_NODE (self));
   eval_mgr      = g_object_new (GEGL_TYPE_EVAL_MGR, NULL);
   eval_mgr->roi = *roi;
   buffer        = gegl_eval_mgr_apply (eval_mgr, self, output_pad_name);
+
   g_object_unref (eval_mgr);
+
   return buffer;
 }
 
 void
-gegl_node_blit (GeglNode      *node,
-                gdouble        scale,
-                GeglRectangle *roi,
-                Babl          *format,
-                gpointer       destination_buf,
-                gint           rowstride,
-                GeglBlitFlags  flags)
+gegl_node_blit (GeglNode            *node,
+                gdouble              scale,
+                const GeglRectangle *roi,
+                const Babl          *format,
+                gpointer             destination_buf,
+                gint                 rowstride,
+                GeglBlitFlags        flags)
 {
+  g_return_if_fail (GEGL_IS_NODE (node));
+  g_return_if_fail (roi != NULL);
+
   if (flags == GEGL_BLIT_DEFAULT)
     {
       GeglBuffer *buffer;
       buffer = gegl_node_apply_roi (node, "output", roi);
-      {
-        GeglBuffer *roi_buf = gegl_buffer_create_sub_buffer (buffer, roi);
+      if (buffer && destination_buf)
+        {
+          GeglBuffer *roi_buf = gegl_buffer_create_sub_buffer (buffer, roi);
+
+          if (destination_buf)
+            {
+              gegl_buffer_get (roi_buf, 1.0, NULL,
+                               format, destination_buf, rowstride);
+            }
+          g_object_unref (roi_buf);
+          if (scale != 1.0)
+            {
+              g_warning ("Scale %f!=1.0 in blit without cache", scale);
+            }
+        }
 
-        if (destination_buf)
-          {
-            gegl_buffer_get (roi_buf, 1.0, NULL, format, destination_buf, rowstride);
-          }
-        g_object_unref (roi_buf);
-        if (scale != 1.0)
-          {
-            g_warning ("Scale %f!=1.0 in blit without cache", scale);
-          }
-      }
       /* and unrefing to ultimately clean it off from the graph */
-      g_object_unref (buffer);
+      if (buffer)
+        g_object_unref (buffer);
     }
   else if ((flags & GEGL_BLIT_CACHE) ||
            (flags & GEGL_BLIT_DIRTY))
@@ -784,7 +833,8 @@ gegl_node_apply (GeglNode    *self,
 {
   GeglRectangle defined;
 
-  g_assert (GEGL_IS_NODE (self));
+  g_return_val_if_fail (GEGL_IS_NODE (self), NULL);
+
   defined = gegl_node_get_bounding_box (self);
   return gegl_node_apply_roi (self, "output", &defined);
 }
@@ -793,7 +843,9 @@ GSList *
 gegl_node_get_depends_on (GeglNode *self)
 {
   GSList *depends_on = NULL;
-  GSList *llink      = self->sources;
+  GSList *llink;
+
+  g_return_val_if_fail (GEGL_IS_NODE (self), NULL);
 
   for (llink = self->sources; llink; llink = g_slist_next (llink))
     {
@@ -819,10 +871,10 @@ gegl_node_get_depends_on (GeglNode *self)
       !strcmp (gegl_node_get_name (self), "proxynop-input"))
     {
       GeglGraph *graph = g_object_get_data (G_OBJECT (self), "graph");
+
       if (graph)
-        {
-          depends_on = g_slist_concat (depends_on, gegl_node_get_depends_on (GEGL_NODE (graph)));
-        }
+        depends_on = g_slist_concat (depends_on,
+                                     gegl_node_get_depends_on (GEGL_NODE (graph)));
     }
 
   return depends_on;
@@ -835,7 +887,7 @@ visitable_accept (GeglVisitable *visitable,
   gegl_visitor_visit_node (visitor, (GeglNode *) visitable);
 }
 
-GSList *
+static GSList *
 visitable_depends_on (GeglVisitable *visitable)
 {
   GeglNode *self = GEGL_NODE (visitable);
@@ -897,9 +949,10 @@ gegl_node_set_op_class (GeglNode    *node,
 }
 
 
-static void property_changed (GObject    *gobject,
-                              GParamSpec *arg1,
-                              gpointer    user_data)
+static void
+property_changed (GObject    *gobject,
+                  GParamSpec *arg1,
+                  gpointer    user_data)
 {
   GeglNode *self = GEGL_NODE (user_data);
 
@@ -1053,8 +1106,6 @@ gegl_node_get (GeglNode    *self,
   va_end (var_args);
 }
 
-#include <stdio.h>
-
 void
 gegl_node_set_valist (GeglNode    *self,
                       const gchar *first_property_name,
@@ -1077,8 +1128,8 @@ gegl_node_set_valist (GeglNode    *self,
 
       if (!strcmp (property_name, "operation"))
         {
-          const gchar *op_class,
-          *op_first_property;
+          const gchar *op_class;
+          const gchar *op_first_property;
 
           op_class          = va_arg (var_args, gchar *);
           op_first_property = va_arg (var_args, gchar *);
@@ -1103,6 +1154,7 @@ gegl_node_set_valist (GeglNode    *self,
               break;
             }
           g_object_set_property (G_OBJECT (self), property_name, &value);
+          g_value_unset (&value);
         }
       else
         {
@@ -1114,7 +1166,8 @@ gegl_node_set_valist (GeglNode    *self,
           if (!pspec)
             {
               g_warning ("%s:%s has no property named: '%s'",
-                         G_STRFUNC, gegl_node_get_debug_name (self), property_name);
+                         G_STRFUNC,
+                         gegl_node_get_debug_name (self), property_name);
               break;
             }
           if (!(pspec->flags & G_PARAM_WRITABLE))
@@ -1136,6 +1189,7 @@ gegl_node_set_valist (GeglNode    *self,
               break;
             }
           g_object_set_property (G_OBJECT (self->operation), property_name, &value);
+          g_value_unset (&value);
         }
 
       property_name = va_arg (var_args, gchar *);
@@ -1199,7 +1253,8 @@ gegl_node_get_valist (GeglNode    *self,
           if (!pspec)
             {
               g_warning ("%s:%s has no property named: '%s'",
-                         G_STRFUNC, gegl_node_get_debug_name (self), property_name);
+                         G_STRFUNC,
+                         gegl_node_get_debug_name (self), property_name);
               break;
             }
           if (!(pspec->flags & G_PARAM_READABLE))
@@ -1233,6 +1288,10 @@ gegl_node_set_property (GeglNode     *self,
                         const gchar  *property_name,
                         const GValue *value)
 {
+  g_return_if_fail (GEGL_IS_NODE (self));
+  g_return_if_fail (property_name != NULL);
+  g_return_if_fail (value != NULL);
+
   if (!strcmp (property_name, "class"))
     {
       g_warning ("Setting a deprecated property \"class\", "
@@ -1262,9 +1321,12 @@ gegl_node_get_property (GeglNode    *self,
                         const gchar *property_name,
                         GValue      *value)
 {
+  g_return_if_fail (GEGL_IS_NODE (self));
+  g_return_if_fail (property_name != NULL);
+  g_return_if_fail (value != NULL);
+
   if (!strcmp (property_name, "operation") ||
-      !strcmp (property_name, "name")
-  )
+      !strcmp (property_name, "name"))
     {
       g_object_get_property (G_OBJECT (self),
                              property_name, value);
@@ -1295,6 +1357,9 @@ gegl_node_find_property (GeglNode    *self,
 {
   GParamSpec *pspec = NULL;
 
+  g_return_val_if_fail (GEGL_IS_NODE (self), NULL);
+  g_return_val_if_fail (property_name != NULL, NULL);
+
   if (self->operation)
     pspec = g_object_class_find_property (
       G_OBJECT_GET_CLASS (G_OBJECT (self->operation)), property_name);
@@ -1305,19 +1370,19 @@ gegl_node_find_property (GeglNode    *self,
 }
 
 const gchar *
-gegl_node_get_operation (GeglNode *node)
+gegl_node_get_operation (const GeglNode *node)
 {
   if (node == NULL)
-    {
-      return NULL;
-    }
-  if (node->is_graph &&
-      node->operation == NULL)
-    return "GraphNode";
+    return NULL;
+
   if (node->operation == NULL)
     {
+      if (node->is_graph)
+        return "GraphNode";
+
       return NULL;
     }
+
   return GEGL_OPERATION_GET_CLASS (node->operation)->name;
 }
 
@@ -1329,26 +1394,43 @@ gegl_node_set_need_rect (GeglNode *node,
                          gint      width,
                          gint      height)
 {
-  GeglNodeDynamic *dynamic = gegl_node_get_dynamic (node, context_id);
+  GeglNodeContext *context;
 
-  g_assert (node);
-  dynamic->need_rect.x      = x;
-  dynamic->need_rect.y      = y;
-  dynamic->need_rect.width  = width;
-  dynamic->need_rect.height = height;
+  g_return_if_fail (GEGL_IS_NODE (node));
+  g_return_if_fail (context_id != NULL);
+
+  context = gegl_node_get_context (node, context_id);
+
+  context->need_rect.x      = x;
+  context->need_rect.y      = y;
+  context->need_rect.width  = width;
+  context->need_rect.height = height;
 }
 
 const gchar *
 gegl_node_get_debug_name (GeglNode *node)
 {
-  static gchar ret_buf[512];
+  static gchar  ret_buf[512];
 
-  if (gegl_node_get_name (node) != NULL &&
-      gegl_node_get_name (node)[0] != '\0')
-    sprintf (ret_buf, "%s named %s", gegl_node_get_operation (node),
-             gegl_node_get_name (node));
+  const gchar  *name;
+  const gchar  *operation;
+
+  g_return_val_if_fail (GEGL_IS_NODE (node), NULL);
+
+  name      = gegl_node_get_name (node);
+  operation = gegl_node_get_operation (node);
+
+  if (name && *name)
+    {
+      g_snprintf (ret_buf, sizeof (ret_buf),
+                  "%s named %s", operation ? operation : "(none)", name);
+    }
   else
-    sprintf (ret_buf, "%s", gegl_node_get_operation (node));
+    {
+      g_snprintf (ret_buf, sizeof (ret_buf),
+                  "%s", operation ? operation : "(none)");
+    }
+
   return ret_buf;
 }
 
@@ -1357,7 +1439,12 @@ gegl_node_get_producer (GeglNode *node,
                         gchar    *pad_name,
                         gchar   **output_pad_name)
 {
-  GeglPad *pad = gegl_node_get_pad (node, pad_name);
+  GeglPad *pad;
+
+  g_return_val_if_fail (GEGL_IS_NODE (node), NULL);
+  g_return_val_if_fail (pad_name != NULL, NULL);
+
+  pad = gegl_node_get_pad (node, pad_name);
 
   if (pad &&
       gegl_pad_is_input (pad) &&
@@ -1382,7 +1469,7 @@ gegl_node_get_bounding_box (GeglNode *root)
   GeglVisitor  *prepare_visitor;
   GeglVisitor  *have_visitor;
   GeglVisitor  *finish_visitor;
-  guchar       *id = g_malloc (1);
+  guchar       *id;
   gint          i;
 
   GeglPad      *pad;
@@ -1398,6 +1485,8 @@ gegl_node_get_bounding_box (GeglNode *root)
     return dummy;
   g_object_ref (root);
 
+  id = g_malloc (1);
+
   for (i = 0; i < 2; i++)
     {
       prepare_visitor = g_object_new (GEGL_TYPE_PREPARE_VISITOR, "id", id, NULL);
@@ -1424,7 +1513,8 @@ gegl_node_process (GeglNode *self)
 {
   GeglProcessor *processor;
 
-  g_assert (GEGL_IS_NODE (self));
+  g_return_if_fail (GEGL_IS_NODE (self));
+
   processor = gegl_node_new_processor (self, NULL);
 
   while (gegl_processor_work (processor, NULL)) ;
@@ -1440,7 +1530,7 @@ void
 gegl_node_process (GeglNode *self)
 {
   GeglNode        *input;
-  GeglNodeDynamic *dynamic;
+  GeglNodeContext *context;
   GeglBuffer      *buffer;
   GeglRectangle    defined;
 
@@ -1453,90 +1543,104 @@ gegl_node_process (GeglNode *self)
   buffer  = gegl_node_apply_roi (input, "output", &defined);
 
   g_assert (GEGL_IS_BUFFER (buffer));
-  dynamic = gegl_node_add_dynamic (self, &defined);
+  context = gegl_node_add_context (self, &defined);
 
   {
     GValue value = { 0, };
     g_value_init (&value, GEGL_TYPE_BUFFER);
     g_value_set_object (&value, buffer);
-    gegl_node_dynamic_set_property (dynamic, "input", &value);
+    gegl_node_context_set_property (context, "input", &value);
     g_value_unset (&value);
   }
 
-  gegl_node_dynamic_set_result_rect (dynamic, defined.x, defined.y, defined.width, defined.h);
+  gegl_node_context_set_result_rect (context, defined.x, defined.y, defined.width, defined.h);
   gegl_operation_process (self->operation, &defined, "foo");
-  gegl_node_remove_dynamic (self, &defined);
+  gegl_node_remove_context (self, &defined);
   g_object_unref (buffer);
 }
 #endif
 
 static gint
-lookup_dynamic (gconstpointer a,
+lookup_context (gconstpointer a,
                 gconstpointer context_id)
 {
-  GeglNodeDynamic *dynamic = (void *) a;
+  GeglNodeContext *context = (void *) a;
 
-  if (dynamic->context_id == context_id)
+  if (context->context_id == context_id)
     return 0;
   return -1;
 }
+
 void babl_backtrack (void);
-GeglNodeDynamic *
-gegl_node_get_dynamic (GeglNode *self,
+
+GeglNodeContext *
+gegl_node_get_context (GeglNode *self,
                        gpointer  context_id)
 {
   GSList          *found;
-  GeglNodeDynamic *dynamic = NULL;
+  GeglNodeContext *context = NULL;
 
-  found = g_slist_find_custom (self->dynamic, context_id, lookup_dynamic);
+  g_return_val_if_fail (GEGL_IS_NODE (self), NULL);
+  g_return_val_if_fail (context_id != NULL, NULL);
+
+  found = g_slist_find_custom (self->context, context_id, lookup_context);
   if (found)
-    dynamic = found->data;
+    context = found->data;
   else
     {
       g_warning ("didn't find %p", context_id);
       babl_backtrack ();
     }
-  return dynamic;
+  return context;
 }
 
 void
-gegl_node_remove_dynamic (GeglNode *self,
+gegl_node_remove_context (GeglNode *self,
                           gpointer  context_id)
 {
-  GeglNodeDynamic *dynamic;
+  GeglNodeContext *context;
 
-  dynamic = gegl_node_get_dynamic (self, context_id);
-  if (!dynamic)
+  g_return_if_fail (GEGL_IS_NODE (self));
+  g_return_if_fail (context_id != NULL);
+
+  context = gegl_node_get_context (self, context_id);
+  if (!context)
     {
-      g_warning ("didn't find dynamic %p for %s", context_id, gegl_node_get_debug_name (self));
+      g_warning ("didn't find context %p for %s",
+                 context_id, gegl_node_get_debug_name (self));
       return;
     }
-  self->dynamic = g_slist_remove (self->dynamic, dynamic);
-  g_object_unref (dynamic);
+  self->context = g_slist_remove (self->context, context);
+  g_object_unref (context);
 }
 
-GeglNodeDynamic *
-gegl_node_add_dynamic (GeglNode *self,
+GeglNodeContext *
+gegl_node_add_context (GeglNode *self,
                        gpointer  context_id)
 {
-  GeglNodeDynamic *dynamic = NULL;
-  GSList          *found   = g_slist_find_custom (self->dynamic, context_id, lookup_dynamic);
+  GeglNodeContext *context = NULL;
+  GSList          *found;
+
+  g_return_val_if_fail (GEGL_IS_NODE (self), NULL);
+  g_return_val_if_fail (context_id != NULL, NULL);
+
+  found = g_slist_find_custom (self->context, context_id, lookup_context);
 
   if (found)
-    dynamic = found->data;
+    context = found->data;
 
-  if (dynamic)
+  if (context)
     {
       /* silently ignore, since multiple traversals of prepare are done
        * to saturate the graph */
-      return dynamic;
+      return context;
     }
 
-  dynamic             = g_object_new (GEGL_TYPE_NODE_DYNAMIC, NULL);
-  dynamic->node       = self;
-  dynamic->context_id = context_id;
-  self->dynamic       = g_slist_prepend (self->dynamic, dynamic);
-  return dynamic;
+  context             = g_object_new (GEGL_TYPE_NODE_CONTEXT, NULL);
+  context->node       = self;
+  context->context_id = context_id;
+  self->context       = g_slist_prepend (self->context, context);
+  return context;
 }
 
 GeglNode *
@@ -1572,8 +1676,14 @@ void
 gegl_node_insert_before (GeglNode *self,
                          GeglNode *to_be_inserted)
 {
-  GeglNode     *other     = gegl_node_get_producer (self, "input", NULL);/*XXX: handle pad name */
-  GeglRectangle rectangle = gegl_node_get_bounding_box (to_be_inserted);
+  GeglNode     *other;
+  GeglRectangle rectangle;
+
+  g_return_if_fail (GEGL_IS_NODE (self));
+  g_return_if_fail (GEGL_IS_NODE (to_be_inserted));
+
+  other     = gegl_node_get_producer (self, "input", NULL);/*XXX: handle pad name */
+  rectangle = gegl_node_get_bounding_box (to_be_inserted);
 
   g_signal_handlers_block_matched (other, G_SIGNAL_MATCH_FUNC, 0, 0, 0, source_invalidated, NULL);
   /* the blocked handler disappears during the relinking */
@@ -1594,17 +1704,15 @@ gegl_node_get_consumers (GeglNode      *node,
   GeglPad *pad;
   gchar  **pasp = NULL;
 
-  if (!node)
-    {
-      g_warning ("%s node==NULL", __FUNCTION__);
-      return 0;
-    }
+  g_return_val_if_fail (GEGL_IS_NODE (node), 0);
+  g_return_val_if_fail (output_pad != NULL, 0);
 
   pad = gegl_node_get_pad (node, output_pad);
 
   if (!pad)
     {
-      g_warning ("%s: no such pad %s for %s", __FUNCTION__, output_pad, gegl_node_get_debug_name (node));
+      g_warning ("%s: no such pad %s for %s",
+                 G_STRFUNC, output_pad, gegl_node_get_debug_name (node));
       return 0;
     }
 
@@ -1658,26 +1766,29 @@ gegl_node_get_consumers (GeglNode      *node,
   return n_connections;
 }
 
-static void computed_event (GeglCache *self,
-                            void      *foo,
-                            void      *user_data)
+static void
+computed_event (GeglCache *self,
+                void      *foo,
+                void      *user_data)
 {
   GeglNode *node = GEGL_NODE (user_data);
 
-  g_signal_emit (node, gegl_node_signals[GEGL_NODE_COMPUTED], 0, foo, NULL, NULL);
+  g_signal_emit (node, gegl_node_signals[COMPUTED], 0, foo, NULL, NULL);
 }
 
 GeglCache *
 gegl_node_get_cache (GeglNode *node)
 {
+  g_return_val_if_fail (GEGL_IS_NODE (node), NULL);
+
   if (!node->cache)
     {
-      GeglPad *pad;
-      Babl *format;
+      GeglPad    *pad;
+      const Babl *format;
 
       pad = gegl_node_get_pad (node, "output");
       g_assert (pad);
-      format = pad->format;
+      format = gegl_pad_get_format (pad);
       if (!format)
         {
           format = babl_format ("RGBA float");
@@ -1694,28 +1805,27 @@ gegl_node_get_cache (GeglNode *node)
   return node->cache;
 }
 
-void
-gegl_node_disable_cache (GeglNode *node)
-{
-  if (node->cache)
-    {
-      g_object_unref (node->cache);
-      node->cache = NULL;
-    }
-}
-
 const gchar *
 gegl_node_get_name (GeglNode *self)
 {
-  GeglNodePriv *priv = GEGL_NODE_GET_PRIVATE (self);
+  GeglNodePrivate *priv;
+
+  g_return_val_if_fail (GEGL_IS_NODE (self), NULL);
+
+  priv = GEGL_NODE_GET_PRIVATE (self);
 
   return priv->name;
 }
 
-void gegl_node_set_name (GeglNode    *self,
-                         const gchar *name)
+void
+gegl_node_set_name (GeglNode    *self,
+                    const gchar *name)
 {
-  GeglNodePriv *priv = GEGL_NODE_GET_PRIVATE (self);
+  GeglNodePrivate *priv;
+
+  g_return_if_fail (GEGL_IS_NODE (self));
+
+  priv = GEGL_NODE_GET_PRIVATE (self);
 
   if (priv->name)
     g_free (priv->name);
@@ -1742,8 +1852,8 @@ GeglNode *
 gegl_node_add_child (GeglNode *self,
                      GeglNode *child)
 {
-  GeglNodePriv *self_priv;
-  GeglNodePriv *child_priv;
+  GeglNodePrivate *self_priv;
+  GeglNodePrivate *child_priv;
 
   g_return_val_if_fail (GEGL_IS_NODE (self), NULL);
   g_return_val_if_fail (GEGL_IS_NODE (child), NULL);
@@ -1763,8 +1873,8 @@ GeglNode *
 gegl_node_remove_child (GeglNode *self,
                         GeglNode *child)
 {
-  GeglNodePriv *self_priv;
-  GeglNodePriv *child_priv;
+  GeglNodePrivate *self_priv;
+  GeglNodePrivate *child_priv;
 
   g_return_val_if_fail (GEGL_IS_NODE (self), NULL);
   g_return_val_if_fail (GEGL_IS_NODE (child), NULL);
@@ -1772,6 +1882,10 @@ gegl_node_remove_child (GeglNode *self,
   self_priv  = GEGL_NODE_GET_PRIVATE (self);
   child_priv = GEGL_NODE_GET_PRIVATE (child);
 
+  g_assert (child_priv->parent == self ||
+            child_priv->parent == NULL);
+  g_return_val_if_fail (GEGL_IS_NODE (child), NULL);
+
   self_priv->children = g_slist_remove (self_priv->children, child);
 
   if (child_priv->parent != NULL)
@@ -1789,10 +1903,48 @@ gegl_node_remove_child (GeglNode *self,
   return child;
 }
 
+GeglNode *
+gegl_node_get_parent (GeglNode *self)
+{
+  GeglNodePrivate *self_priv;
+
+  g_return_val_if_fail (GEGL_IS_NODE (self), NULL);
+
+  self_priv  = GEGL_NODE_GET_PRIVATE (self);
+  return self_priv->parent;
+}
+
+GeglNode *
+gegl_node_adopt_child (GeglNode *self,
+                       GeglNode *child)
+{
+  GeglNode *old_parent;
+
+  g_return_val_if_fail (GEGL_IS_NODE (child), NULL);
+  g_object_ref (child);
+  old_parent = gegl_node_get_parent (child);
+  if (old_parent)
+    {
+      gegl_node_remove_child (old_parent, child);
+    }
+
+  if (self)
+    {
+      gegl_node_add_child (self, child);
+    }
+  else
+    {
+      g_object_ref (child);
+    }
+
+  g_object_unref (child);
+  return child;
+}
+
 gint
 gegl_node_get_num_children (GeglNode *self)
 {
-  GeglNodePriv *priv;
+  GeglNodePrivate *priv;
 
   g_return_val_if_fail (GEGL_IS_NODE (self), -1);
 
@@ -1805,7 +1957,7 @@ GeglNode *
 gegl_node_get_nth_child (GeglNode *self,
                          gint      n)
 {
-  GeglNodePriv *priv;
+  GeglNodePrivate *priv;
 
   g_return_val_if_fail (GEGL_IS_NODE (self), NULL);
 
@@ -1820,7 +1972,7 @@ gegl_node_get_nth_child (GeglNode *self,
 GSList *
 gegl_node_get_children (GeglNode *self)
 {
-  GeglNodePriv *priv;
+  GeglNodePrivate *priv;
 
   g_return_val_if_fail (GEGL_IS_NODE (self), NULL);
 
@@ -1833,59 +1985,53 @@ gegl_node_get_children (GeglNode *self)
  *  returns a freshly created node, owned by the graph, and thus freed with it
  */
 GeglNode *
-gegl_node_new_child (GeglNode    *node,
+gegl_node_new_child (GeglNode    *parent,
                      const gchar *first_property_name,
                      ...)
 {
-  GeglNode    *self = node;
+  GeglNode    *node;
   va_list      var_args;
   const gchar *name;
 
-  g_return_val_if_fail (GEGL_IS_NODE (self), NULL);
-
   node = g_object_new (GEGL_TYPE_NODE, NULL);
-  gegl_node_add_child (self, node);
+  if (parent)
+    gegl_node_add_child (parent, node);
 
   name = first_property_name;
   va_start (var_args, first_property_name);
   gegl_node_set_valist (node, first_property_name, var_args);
   va_end (var_args);
 
-  g_object_unref (node);
+  if (parent)
+    g_object_unref (node);
   return node;
 }
 
-GeglNode *gegl_node_create_child (GeglNode    *self,
-                                  const gchar *operation)
+GeglNode *
+gegl_node_create_child (GeglNode    *self,
+                        const gchar *operation)
 {
+  g_return_val_if_fail (operation != NULL, NULL);
+
   return gegl_node_new_child (self, "operation", operation, NULL);
 }
 
 static void
-graph_source_invalidated (GeglNode      *source,
-                          GeglRectangle *rect,
-                          gpointer       data)
+graph_source_invalidated (GeglNode            *source,
+                          const GeglRectangle *rect,
+                          gpointer             data)
 {
-  GeglRectangle dirty_rect;
-  GeglNode     *destination = GEGL_NODE (data);
-  gchar        *source_name;
-  gchar        *destination_name;
-
-  destination_name = g_strdup (gegl_node_get_debug_name (destination));
-  source_name      = g_strdup (gegl_node_get_debug_name (source));
+  GeglNode      *destination = GEGL_NODE (data);
+  GeglRectangle  dirty_rect  = *rect;
 
   if (0) g_warning ("graph:%s is dirtied from %s (%i,%i %ix%i)",
-                    destination_name,
-                    source_name,
+                    gegl_node_get_debug_name (destination),
+                    gegl_node_get_debug_name (source),
                     rect->x, rect->y,
                     rect->width, rect->height);
 
-  dirty_rect = *rect;
-
-  g_signal_emit (destination, gegl_node_signals[GEGL_NODE_INVALIDATED], 0, &dirty_rect, NULL);
-
-  g_free (source_name);
-  g_free (destination_name);
+  g_signal_emit (destination, gegl_node_signals[INVALIDATED], 0,
+                 &dirty_rect, NULL);
 }
 
 
@@ -1938,6 +2084,8 @@ GeglNode *
 gegl_node_get_input_proxy (GeglNode    *node,
                            const gchar *name)
 {
+  g_return_val_if_fail (GEGL_IS_NODE (node), NULL);
+
   return gegl_node_get_pad_proxy (node, name, TRUE);
 }
 
@@ -1945,6 +2093,8 @@ GeglNode *
 gegl_node_get_output_proxy (GeglNode    *node,
                             const gchar *name)
 {
+  g_return_val_if_fail (GEGL_IS_NODE (node), NULL);
+
   return gegl_node_get_pad_proxy (node, name, FALSE);
 }
 
