@@ -15,18 +15,28 @@
  *
  * Copyright 2006 Øyvind Kolås
  */
+
+
+#define GEGL_INTERNAL
+#include "config.h"
+
+#include <glib-object.h>
+#include "gegl-types.h"
 #include "gegl-operation-point-composer.h"
 #include "gegl-utils.h"
+#include "graph/gegl-node.h"
 #include "graph/gegl-pad.h"
 #include <string.h>
 
-static gboolean process_inner (GeglOperation *operation,
-                               gpointer       context_id);
+static gboolean process_inner (GeglOperation       *operation,
+                               GeglBuffer          *input,
+                               GeglBuffer          *aux,
+                               GeglBuffer          *output,
+                               const GeglRectangle *result);
 
 G_DEFINE_TYPE (GeglOperationPointComposer, gegl_operation_point_composer, GEGL_TYPE_OPERATION_COMPOSER)
 
-static void prepare (GeglOperation *operation,
-                     gpointer       context_id)
+static void prepare (GeglOperation *operation)
 {
   Babl *format = babl_format ("RGBA float");
   gegl_operation_set_format (operation, "input", format);
@@ -52,36 +62,40 @@ gegl_operation_point_composer_init (GeglOperationPointComposer *self)
 
 }
 
+#if 0 /* FIXME: this should be re-enabled, possibly by skipping the point-composer class duplicating that
+       * code and directly implement on top of GeglOperation
+       */
 static gboolean
-fast_paths (GeglOperation *operation,
-            gpointer       context_id,
-            Babl          *in_format,
-            Babl          *aux_format,
-            Babl          *out_format);
+fast_paths (GeglOperation       *operation,
+            GeglNodeContext     *context,
+            const Babl          *in_format,
+            const Babl          *aux_format,
+            const Babl          *out_format,
+            const GeglRectangle *result);
+#endif
 
 static gboolean
-process_inner (GeglOperation *operation,
-               gpointer       context_id)
+process_inner (GeglOperation       *operation,
+               GeglBuffer          *input,
+               GeglBuffer          *aux,
+               GeglBuffer          *output,
+               const GeglRectangle *result)
 {
-  GeglBuffer    *input = GEGL_BUFFER (gegl_operation_get_data (operation, context_id, "input"));
-  GeglBuffer    *aux   = GEGL_BUFFER (gegl_operation_get_data (operation, context_id, "aux"));
-  GeglRectangle *result = gegl_operation_result_rect (operation, context_id);
-  GeglBuffer    *output;
-  GeglPad       *pad;
-  Babl          *in_format;
-  Babl          *aux_format;
-  Babl          *out_format;
+  GeglPad    *pad;
+  const Babl *in_format;
+  const Babl *aux_format;
+  const Babl *out_format;
 
   pad       = gegl_node_get_pad (operation->node, "input");
-  in_format = pad->format;
+  in_format = gegl_pad_get_format (pad);
   if (!in_format)
     {
       g_warning ("%s", gegl_node_get_debug_name (operation->node));
     }
   g_assert (in_format);
 
-  pad       = gegl_node_get_pad (operation->node, "aux");
-  aux_format = pad->format;
+  pad        = gegl_node_get_pad (operation->node, "aux");
+  aux_format = gegl_pad_get_format (pad);
   if (!aux_format)
     {
       g_warning ("%s", gegl_node_get_debug_name (operation->node));
@@ -89,7 +103,7 @@ process_inner (GeglOperation *operation,
   g_assert (aux_format);
 
   pad        = gegl_node_get_pad (operation->node, "output");
-  out_format = pad->format;
+  out_format = gegl_pad_get_format (pad);
   if (!out_format)
     {
       g_warning ("%s", gegl_node_get_debug_name (operation->node));
@@ -102,11 +116,19 @@ process_inner (GeglOperation *operation,
    * good idea. NB! some of the OpenRaster meta ops, depends on the
    * short-circuiting happening in fast_paths.
    * */
-  if (fast_paths (operation, context_id, in_format, aux_format, out_format))
+#if 0
+  if (0 && fast_paths (operation, context,
+                       in_format,
+                       aux_format,
+                       out_format,
+                       result))
     return TRUE;
+#endif
 
+#if 0
   /* retrieve the buffer we're writing to from GEGL */
-  output = gegl_operation_get_target (operation, context_id, "output");
+  output = gegl_node_context_get_target (context, "output");
+#endif
 
   if ((result->width > 0) && (result->height > 0))
     {
@@ -141,7 +163,7 @@ process_inner (GeglOperation *operation,
           output->extent.width * output->extent.height);
       }
 
-      gegl_buffer_set (output, NULL, out_format, out_buf);
+      gegl_buffer_set (output, NULL, out_format, out_buf, GEGL_AUTO_ROWSTRIDE);
 
       g_free (in_buf);
       if (in_format != out_format)
@@ -152,22 +174,22 @@ process_inner (GeglOperation *operation,
   return TRUE;
 }
 
-
+#if 0
 static gboolean
-fast_paths (GeglOperation *operation,
-            gpointer       context_id,
-            Babl          *in_format,
-            Babl          *aux_format,
-            Babl          *out_format)
+fast_paths (GeglOperation       *operation,
+            GeglNodeContext     *context,
+            const Babl          *in_format,
+            const Babl          *aux_format,
+            const Babl          *out_format,
+            const GeglRectangle *result)
 {
-  GeglBuffer    *input = GEGL_BUFFER (gegl_operation_get_data (operation, context_id, "input"));
-  GeglBuffer    *aux   = GEGL_BUFFER (gegl_operation_get_data (operation, context_id, "aux"));
-  GeglRectangle *result = gegl_operation_result_rect (operation, context_id);
+  GeglBuffer  *input = gegl_node_context_get_source (context, "input");
+  GeglBuffer  *aux   = gegl_node_context_get_source (context, "aux");
 
   if (!input && aux)
     {
       g_object_ref (aux);
-      gegl_operation_set_data (operation, context_id, "output", G_OBJECT (aux));
+      gegl_node_context_set_object (context, "output", G_OBJECT (aux));
       return TRUE;
     }
 
@@ -195,11 +217,11 @@ fast_paths (GeglOperation *operation,
                   if (!gegl_rectangle_intersect (NULL, aux_abyss, result))
                     {
                       GeglBuffer *output = gegl_buffer_new (NULL, NULL);
-                      gegl_operation_set_data (operation, context_id, "output", G_OBJECT (output));
+                      gegl_node_context_set_object (context, "output", G_OBJECT (output));
                       return TRUE;
                     }
                   g_object_ref (aux);
-                  gegl_operation_set_data (operation, context_id, "output", G_OBJECT (aux));
+                  gegl_node_context_set_object (context, "output", G_OBJECT (aux));
                   return TRUE;
                 }
             }
@@ -214,7 +236,7 @@ fast_paths (GeglOperation *operation,
                   (aux && !gegl_rectangle_intersect (NULL, aux_abyss, result)))
                 {
                   g_object_ref (input);
-                  gegl_operation_set_data (operation, context_id, "output", G_OBJECT (input));
+                  gegl_node_context_set_object (context, "output", G_OBJECT (input));
                   return TRUE;
                 }
             }
@@ -223,9 +245,10 @@ fast_paths (GeglOperation *operation,
     else
       {
         GeglBuffer *output = gegl_buffer_new (NULL, out_format);
-        gegl_operation_set_data (operation, context_id, "output", G_OBJECT (output));
+        gegl_node_context_set_object (context, "output", G_OBJECT (output));
         return TRUE;
       }
   }
   return FALSE;
 }
+#endif
