@@ -15,37 +15,35 @@
  *
  * Copyright 2006 Øyvind Kolås
  */
-#ifndef _GEGL_OPERATION_POINT_COMPOSER_H
-#define _GEGL_OPERATION_POINT_COMPOSER_H
 
-#include <glib-object.h>
-#include "gegl-types.h"
-#include "buffer/gegl-buffer.h"
-#include "gegl-operation.h"
+#ifndef __GEGL_OPERATION_POINT_COMPOSER_H__
+#define __GEGL_OPERATION_POINT_COMPOSER_H__
+
 #include "gegl-operation-composer.h"
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
+G_BEGIN_DECLS
 
-#define GEGL_TYPE_OPERATION_POINT_COMPOSER           (gegl_operation_point_composer_get_type ())
-#define GEGL_OPERATION_POINT_COMPOSER(obj)           (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEGL_TYPE_OPERATION_POINT_COMPOSER, GeglOperationPointComposer))
-#define GEGL_OPERATION_POINT_COMPOSER_CLASS(klass)   (G_TYPE_CHECK_CLASS_CAST ((klass),  GEGL_TYPE_OPERATION_POINT_COMPOSER, GeglOperationPointComposerClass))
-#define GEGL_OPERATION_POINT_COMPOSER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj),  GEGL_TYPE_OPERATION_POINT_COMPOSER, GeglOperationPointComposerClass))
+#define GEGL_TYPE_OPERATION_POINT_COMPOSER            (gegl_operation_point_composer_get_type ())
+#define GEGL_OPERATION_POINT_COMPOSER(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEGL_TYPE_OPERATION_POINT_COMPOSER, GeglOperationPointComposer))
+#define GEGL_OPERATION_POINT_COMPOSER_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass),  GEGL_TYPE_OPERATION_POINT_COMPOSER, GeglOperationPointComposerClass))
+#define GEGL_IS_OPERATION_POINT_COMPOSER(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEGL_TYPE_OPERATION_POINT_COMPOSER))
+#define GEGL_IS_OPERATION_POINT_COMPOSER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass),  GEGL_TYPE_OPERATION_POINT_COMPOSER))
+#define GEGL_OPERATION_POINT_COMPOSER_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj),  GEGL_TYPE_OPERATION_POINT_COMPOSER, GeglOperationPointComposerClass))
 
 typedef struct _GeglOperationPointComposer  GeglOperationPointComposer;
 struct _GeglOperationPointComposer
 {
-    GeglOperationComposer  operation;
-    gint                 samples;
-    /*< private >*/
+  GeglOperationComposer parent_instance;
+
+  /*< private >*/
 };
 
 typedef struct _GeglOperationPointComposerClass GeglOperationPointComposerClass;
 struct _GeglOperationPointComposerClass
 {
-   GeglOperationComposerClass operation_class;
-   gboolean (*process) (GeglOperation *self,      /* for parameters      */
+  GeglOperationComposerClass parent_class;
+
+  gboolean (* process) (GeglOperation *self,      /* for parameters      */
                         void          *in,
                         void          *aux,
                         void          *out,
@@ -55,8 +53,6 @@ struct _GeglOperationPointComposerClass
 
 GType gegl_operation_point_composer_get_type (void) G_GNUC_CONST;
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 #endif
