@@ -15,7 +15,14 @@
  *
  * Copyright 2006 Øyvind Kolås
  */
+
+#define GEGL_INTERNAL
+#include "config.h"
+
+#include <glib-object.h>
+#include "gegl-types.h"
 #include "gegl-operation-source.h"
+#include "graph/gegl-node.h"
 #include "graph/gegl-pad.h"
 #include <string.h>
 
@@ -35,21 +42,21 @@ static void     set_property (GObject      *gobject,
                               const GValue *value,
                               GParamSpec   *pspec);
 static gboolean process      (GeglOperation *operation,
-                              gpointer       context_id,
-                              const gchar   *output_prop);
+                              GeglNodeContext *context,
+                              const gchar   *output_prop,
+                              const GeglRectangle *result);
 static void     attach       (GeglOperation *operation);
 
 
 
 G_DEFINE_TYPE (GeglOperationSource, gegl_operation_source, GEGL_TYPE_OPERATION)
 
-static GeglRectangle get_defined_region (GeglOperation *self);
-static GeglRectangle compute_input_request (GeglOperation *operation,
-                                            const gchar   *input_pad,
-                                            GeglRectangle *roi);
-static GeglRectangle  adjust_result_region (GeglOperation *operation,
-                                            GeglRectangle *roi);
-
+static GeglRectangle get_bounding_box          (GeglOperation        *self);
+static GeglRectangle get_required_for_output   (GeglOperation        *operation,
+                                                 const gchar         *input_pad,
+                                                 const GeglRectangle *roi);
+static GeglRectangle  get_cached_region        (GeglOperation       *operation,
+                                                 const GeglRectangle *roi);
 
 
 static void
@@ -63,10 +70,10 @@ gegl_operation_source_class_init (GeglOperationSourceClass * klass)
 
   operation_class->process = process;
   operation_class->attach  = attach;
-  operation_class->adjust_result_region = adjust_result_region;
+  operation_class->get_cached_region = get_cached_region;
 
-  operation_class->get_defined_region  = get_defined_region;
-  operation_class->compute_input_request = compute_input_request;
+  operation_class->get_bounding_box  = get_bounding_box;
+  operation_class->get_required_for_output = get_required_for_output;
 
   g_object_class_install_property (gobject_class, PROP_OUTPUT,
                                    g_param_spec_object ("output",
@@ -74,7 +81,7 @@ gegl_operation_source_class_init (GeglOperationSourceClass * klass)
                                                         "Ouput pad for generated image buffer.",
                                                         GEGL_TYPE_BUFFER,
                                                         G_PARAM_READABLE |
-                                                        GEGL_PAD_OUTPUT));
+                                                        GEGL_PARAM_PAD_OUTPUT));
 }
 
 static void
@@ -110,11 +117,13 @@ set_property (GObject      *object,
 }
 
 static gboolean
-process (GeglOperation *operation,
-         gpointer       context_id,
-         const gchar   *output_prop)
+process (GeglOperation       *operation,
+         GeglNodeContext     *context,
+         const gchar         *output_prop,
+         const GeglRectangle *result)
 {
   GeglOperationSourceClass *klass = GEGL_OPERATION_SOURCE_GET_CLASS (operation);
+  GeglBuffer               *output;
   gboolean                  success;
 
   if (strcmp (output_prop, "output"))
@@ -124,13 +133,14 @@ process (GeglOperation *operation,
     }
 
   g_assert (klass->process);
-  success = klass->process (operation, context_id);
+  output = gegl_node_context_get_target (context, "output");
+  success = klass->process (operation, output, result);
 
   return success;
 }
 
 static GeglRectangle
-get_defined_region (GeglOperation *self)
+get_bounding_box (GeglOperation *self)
 {
   GeglRectangle result = { 0, 0, 0, 0 };
 
@@ -139,16 +149,17 @@ get_defined_region (GeglOperation *self)
   return result;
 }
 
-static GeglRectangle compute_input_request (GeglOperation *operation,
-                                            const gchar   *input_pad,
-                                            GeglRectangle *roi)
+static GeglRectangle
+get_required_for_output (GeglOperation        *operation,
+                         const gchar         *input_pad,
+                         const GeglRectangle *roi)
 {
   return *roi;
 }
 
 static GeglRectangle
-adjust_result_region (GeglOperation *operation,
-                       GeglRectangle *roi)
+get_cached_region (GeglOperation       *operation,
+                   const GeglRectangle *roi)
 {
   GeglRectangle result = *roi;
   result = operation->node->have_rect;
