@@ -56,8 +56,9 @@ struct _GeglProcessor
   GeglNode        *node;
   GeglRectangle    rectangle;
   GeglNode        *input;
-  GeglNodeDynamic *dynamic;
+  GeglNodeContext *context;
 
+  GeglRegion *valid_region; /* used when doing unbuffered rendering */
   GeglRegion *queued_region;
   GSList     *dirty_rectangles;
   gint        chunk_size;
@@ -101,7 +102,7 @@ static void gegl_processor_init (GeglProcessor *processor)
 {
   processor->node             = NULL;
   processor->input            = NULL;
-  processor->dynamic          = NULL;
+  processor->context          = NULL;
   processor->queued_region    = NULL;
   processor->dirty_rectangles = NULL;
   processor->chunk_size       = 128 * 128;
@@ -123,10 +124,15 @@ static GObject *constructor (GType                  type,
                    GEGL_TYPE_OPERATION_SINK))
     {
       processor->input = gegl_node_get_producer (processor->node, "input", NULL);
+      if (!gegl_operation_sink_needs_full (processor->node->operation))
+        processor->valid_region = gegl_region_new ();
+      else
+        processor->valid_region = NULL;
     }
   else
     {
       processor->input = processor->node;
+      processor->valid_region = NULL;
     }
   g_object_ref (processor->input);
 
@@ -146,6 +152,8 @@ static void finalize (GObject *self_object)
     g_object_unref (processor->input);
   if (processor->queued_region)
     gegl_region_destroy (processor->queued_region);
+  if (processor->valid_region)
+    gegl_region_destroy (processor->valid_region);
 
   G_OBJECT_CLASS (gegl_processor_parent_class)->finalize (self_object);
 }
@@ -207,92 +215,99 @@ get_property (GObject    *gobject,
 }
 
 void
-gegl_processor_set_rectangle (GeglProcessor *processor,
-                              GeglRectangle *rectangle)
+gegl_processor_set_rectangle (GeglProcessor       *processor,
+                              const GeglRectangle *rectangle)
 {
-  GeglRectangle bounds;
+  GSList        *iter;
+  GeglRectangle  bounds;
 
   if (gegl_rectangle_equal (&processor->rectangle, rectangle))
-    {
-      return;
-    }
+    return;
 
   bounds               = gegl_node_get_bounding_box (processor->input);
   processor->rectangle = *rectangle;
   gegl_rectangle_intersect (&processor->rectangle, &processor->rectangle, &bounds);
 
   /* remove already queued dirty rectangles */
-  while (processor->dirty_rectangles)
-    {
-      GeglRectangle *rect = processor->dirty_rectangles->data;
-      processor->dirty_rectangles = g_slist_remove (processor->dirty_rectangles, rect);
-      g_free (rect);
-    }
+  for (iter = processor->dirty_rectangles; iter; iter = g_slist_next (iter))
+    g_slice_free (GeglRectangle, iter->data);
+  g_slist_free (processor->dirty_rectangles);
+  processor->dirty_rectangles = NULL;
 }
 
 GeglProcessor *
-gegl_node_new_processor (GeglNode      *node,
-                         GeglRectangle *rectangle)
+gegl_node_new_processor (GeglNode            *node,
+                         const GeglRectangle *rectangle)
 {
   GeglProcessor *processor;
-  GeglCache     *cache;
 
-  g_assert (GEGL_IS_NODE (node));
+  g_return_val_if_fail (GEGL_IS_NODE (node), NULL);
 
   processor = g_object_new (GEGL_TYPE_PROCESSOR, "node", node, NULL);
 
-
   if (rectangle)
-    gegl_processor_set_rectangle (processor, rectangle);
+    {
+      gegl_processor_set_rectangle (processor, rectangle);
+    }
   else
     {
       GeglRectangle tmp = gegl_node_get_bounding_box (processor->input);
       gegl_processor_set_rectangle (processor, &tmp);
     }
 
-  cache = gegl_node_get_cache (processor->input);
   if (node->operation &&
-      g_type_is_a (G_OBJECT_TYPE (node->operation),
-                   GEGL_TYPE_OPERATION_SINK))
+      GEGL_IS_OPERATION_SINK (node->operation))
     {
-      processor->dynamic = gegl_node_add_dynamic (node, cache);
+      GeglCache     *cache;
+
+      if (!gegl_operation_sink_needs_full (node->operation))
+          return processor;
+      cache = gegl_node_get_cache (processor->input);
+
+      processor->context = gegl_node_add_context (node, cache);
       {
         GValue value = { 0, };
         g_value_init (&value, GEGL_TYPE_BUFFER);
         g_value_set_object (&value, cache);
-        gegl_node_dynamic_set_property (processor->dynamic, "input", &value);
+        gegl_node_context_set_property (processor->context, "input", &value);
         g_value_unset (&value);
       }
 
-      gegl_node_dynamic_set_result_rect (processor->dynamic,
+      gegl_node_context_set_result_rect (processor->context,
                                          processor->rectangle.x,
                                          processor->rectangle.y,
                                          processor->rectangle.width,
                                          processor->rectangle.height);
+      gegl_node_context_set_need_rect   (processor->context,
+                                         processor->rectangle.x,
+                                         processor->rectangle.y,
+                                         processor->rectangle.width,
+                                         processor->rectangle.height);
+
+
     }
   else
     {
-      processor->dynamic = NULL;
+      processor->context = NULL;
     }
 
   return processor;
 }
 
-/* returns TRUE if there is more work */
-static gboolean render_rectangle (GeglProcessor *processor)
-{
-  GeglCache     *cache    = gegl_node_get_cache (processor->input);
-  gint           max_area = processor->chunk_size;
 
-  GeglRectangle *dr;
+
+/* returns TRUE if there is more work */
+static gboolean render_rectangle_buffered (GeglProcessor *processor)
+{
+  GeglCache *cache    = gegl_node_get_cache (processor->input);
+  gint       max_area = processor->chunk_size;
 
   if (processor->dirty_rectangles)
     {
-      guchar *buf;
-      gint pxsize;
-      g_object_get (cache, "px-size", &pxsize, NULL);
+      GeglRectangle *dr = processor->dirty_rectangles->data;
+      gint           pxsize;
 
-      dr = processor->dirty_rectangles->data;
+      g_object_get (cache, "px-size", &pxsize, NULL);
 
       if (dr->height * dr->width > max_area && 1)
         {
@@ -300,13 +315,14 @@ static gboolean render_rectangle (GeglProcessor *processor)
 
           if (dr->height > dr->width)
             {
+              GeglRectangle *fragment;
+
               band_size = dr->height / 2;
 
               if (band_size < 1)
                 band_size = 1;
 
-              GeglRectangle *fragment = g_malloc (sizeof (GeglRectangle));
-              *fragment = *dr;
+              fragment = g_slice_dup (GeglRectangle, dr);
 
               fragment->height = band_size;
               dr->height      -= band_size;
@@ -317,13 +333,14 @@ static gboolean render_rectangle (GeglProcessor *processor)
             }
           else
             {
+              GeglRectangle *fragment;
+
               band_size = dr->width / 2;
 
               if (band_size < 1)
                 band_size = 1;
 
-              GeglRectangle *fragment = g_malloc (sizeof (GeglRectangle));
-              *fragment = *dr;
+              fragment = g_slice_dup (GeglRectangle, dr);
 
               fragment->width = band_size;
               dr->width      -= band_size;
@@ -338,34 +355,112 @@ static gboolean render_rectangle (GeglProcessor *processor)
 
       if (!dr->width || !dr->height)
         {
-          g_free (dr);
+          g_slice_free (GeglRectangle, dr);
           return TRUE;
         }
 
-      if (gegl_region_rect_in (cache->valid_region, (GeglRectangle *) dr) !=
+      if (gegl_region_rect_in (cache->valid_region, dr) !=
           GEGL_OVERLAP_RECTANGLE_IN)
         {
-          gegl_region_union_with_rect (cache->valid_region, (GeglRectangle *) dr);
+          guchar *buf;
+
+          gegl_region_union_with_rect (cache->valid_region, dr);
           buf = g_malloc (dr->width * dr->height * pxsize);
           g_assert (buf);
 
           gegl_node_blit (cache->node, 1.0, dr, cache->format, buf, GEGL_AUTO_ROWSTRIDE, GEGL_BLIT_DEFAULT);
-          gegl_buffer_set (GEGL_BUFFER (cache), dr, cache->format, buf);
+          gegl_buffer_set (GEGL_BUFFER (cache), dr, cache->format, buf, GEGL_AUTO_ROWSTRIDE);
 
-          gegl_cache_computed (cache, (GeglRectangle *) dr);
+          gegl_cache_computed (cache, dr);
 
           g_free (buf);
         }
-/*      else if (gegl_region_rect_in (cache->valid_region, (GeglRectangle *) dr) ==
-          GEGL_OVERLAP_RECTANGLE_PART)
-        {
-          g_warning ("part");
-        }*/
-      g_free (dr);
+
+      g_slice_free (GeglRectangle, dr);
     }
+
   return processor->dirty_rectangles != NULL;
 }
 
+/* returns TRUE if there is more work */
+static gboolean render_rectangle_unbuffered (GeglProcessor *processor)
+{
+  const gint max_area = processor->chunk_size;
+
+  if (processor->dirty_rectangles)
+    {
+      GeglRectangle *dr = processor->dirty_rectangles->data;
+
+      if (dr->height * dr->width > max_area && 1)
+        {
+          gint band_size;
+
+          if (dr->height > dr->width)
+            {
+              GeglRectangle *fragment;
+
+              band_size = dr->height / 2;
+
+              if (band_size < 1)
+                band_size = 1;
+
+              fragment = g_slice_dup (GeglRectangle, dr);
+
+              fragment->height = band_size;
+              dr->height      -= band_size;
+              dr->y           += band_size;
+
+              processor->dirty_rectangles = g_slist_prepend (processor->dirty_rectangles, fragment);
+              return TRUE;
+            }
+          else
+            {
+              GeglRectangle *fragment;
+
+              band_size = dr->width / 2;
+
+              if (band_size < 1)
+                band_size = 1;
+
+              fragment = g_slice_dup (GeglRectangle, dr);
+
+              fragment->width = band_size;
+              dr->width      -= band_size;
+              dr->x          += band_size;
+
+              processor->dirty_rectangles = g_slist_prepend (processor->dirty_rectangles, fragment);
+              return TRUE;
+            }
+        }
+      processor->dirty_rectangles = g_slist_remove (processor->dirty_rectangles, dr);
+
+      if (!dr->width || !dr->height)
+        {
+          g_slice_free (GeglRectangle, dr);
+          return TRUE;
+        }
+
+      gegl_node_blit (processor->node, 1.0, dr, NULL, NULL, GEGL_AUTO_ROWSTRIDE, GEGL_BLIT_DEFAULT);
+      gegl_region_union_with_rect (processor->valid_region, dr);
+      g_slice_free (GeglRectangle, dr);
+    }
+
+  return processor->dirty_rectangles != NULL;
+}
+
+
+/* FIXME: merge duplicated code between the two different code paths
+ */
+static gboolean render_rectangle (GeglProcessor *processor)
+{
+  if (GEGL_IS_OPERATION_SINK (processor->node->operation) &&
+      !gegl_operation_sink_needs_full (processor->node->operation))
+   {
+     return render_rectangle_unbuffered (processor);
+   }
+  return render_rectangle_buffered (processor);
+}
+
 static gint rect_area (GeglRectangle *rectangle)
 {
   return rectangle->width * rectangle->height;
@@ -385,17 +480,18 @@ static gint region_area (GeglRegion *region)
       sum += rect_area (&rectangles[i]);
     }
   g_free (rectangles);
+
   return sum;
 }
 
-static gint area_left (GeglCache     *cache,
+static gint area_left (GeglRegion    *area,
                        GeglRectangle *rectangle)
 {
   GeglRegion *region;
   gint        sum = 0;
 
   region = gegl_region_rectangle (rectangle);
-  gegl_region_subtract (region, cache->valid_region);
+  gegl_region_subtract (region, area);
   sum += region_area (region);
   gegl_region_destroy (region);
   return sum;
@@ -413,13 +509,18 @@ gegl_processor_is_rendered (GeglProcessor *processor)
 static gdouble
 gegl_processor_progress (GeglProcessor *processor)
 {
-  GeglCache *cache = gegl_node_get_cache (processor->input);
+  GeglRegion *valid_region;
   gint valid;
   gint wanted;
   gdouble ret;
 
+  if (processor->valid_region)
+    valid_region = processor->valid_region;
+  else
+    valid_region = gegl_node_get_cache (processor->input)->valid_region;
+
   wanted = rect_area (&(processor->rectangle));
-  valid  = wanted - area_left (cache, &(processor->rectangle));
+  valid  = wanted - area_left (valid_region, &(processor->rectangle));
   if (wanted == 0)
     {
       if (gegl_processor_is_rendered (processor))
@@ -441,9 +542,13 @@ gegl_processor_render (GeglProcessor *processor,
                        GeglRectangle *rectangle,
                        gdouble       *progress)
 {
-  GeglCache *cache = gegl_node_get_cache (processor->input);
+  GeglRegion *valid_region;
+
+  if (processor->valid_region)
+    valid_region = processor->valid_region;
+  else
+    valid_region = gegl_node_get_cache (processor->input)->valid_region;
 
-  g_assert (GEGL_IS_CACHE (cache));
   {
     gboolean more_work = render_rectangle (processor);
     if (more_work == TRUE)
@@ -455,11 +560,11 @@ gegl_processor_render (GeglProcessor *processor,
             if (rectangle)
               {
                 wanted = rect_area (rectangle);
-                valid  = wanted - area_left (cache, rectangle);
+                valid  = wanted - area_left (valid_region, rectangle);
               }
             else
               {
-                valid  = region_area (cache->valid_region);
+                valid  = region_area (valid_region);
                 wanted = region_area (processor->queued_region);
               }
             if (wanted == 0)
@@ -480,30 +585,31 @@ gegl_processor_render (GeglProcessor *processor,
     { /* we're asked to work on a specific rectangle thus we only focus
          on it */
       GeglRegion    *region = gegl_region_rectangle (rectangle);
-      gegl_region_subtract (region, cache->valid_region);
       GeglRectangle *rectangles;
       gint           n_rectangles;
       gint           i;
 
+      gegl_region_subtract (region, valid_region);
       gegl_region_get_rectangles (region, &rectangles, &n_rectangles);
+      gegl_region_destroy (region);
 
       for (i = 0; i < n_rectangles && i < 1; i++)
         {
-          GeglRectangle  roi = *((GeglRectangle *) &rectangles[i]);
-          GeglRectangle *dr;
-          GeglRegion    *tr = gegl_region_rectangle ((void *) &roi);
+          GeglRectangle  roi = rectangles[i];
+          GeglRegion    *tr = gegl_region_rectangle (&roi);
           gegl_region_subtract (processor->queued_region, tr);
           gegl_region_destroy (tr);
 
-          dr                          = g_malloc (sizeof (GeglRectangle));
-          *dr                         = roi;
-          processor->dirty_rectangles = g_slist_prepend (processor->dirty_rectangles, dr);
+          processor->dirty_rectangles = g_slist_prepend (processor->dirty_rectangles,
+                                                         g_slice_dup (GeglRectangle, &roi));
         }
       g_free (rectangles);
+
       if (n_rectangles != 0)
         {
           if (progress)
-            *progress = 1.0 - ((double) area_left (cache, rectangle) / rect_area (rectangle));
+            *progress = 1.0 - ((double) area_left (valid_region, rectangle) /
+                               rect_area (rectangle));
           return TRUE;
         }
       return FALSE;
@@ -517,21 +623,18 @@ gegl_processor_render (GeglProcessor *processor,
       gint           n_rectangles;
       gint           i;
 
-      g_warning ("hey!");
-
-      gegl_region_get_rectangles (processor->queued_region, &rectangles, &n_rectangles);
+      gegl_region_get_rectangles (processor->queued_region, &rectangles,
+                                  &n_rectangles);
 
       for (i = 0; i < n_rectangles && i < 1; i++)
         {
-          GeglRectangle  roi = *((GeglRectangle *) &rectangles[i]);
-          GeglRectangle *dr;
-          GeglRegion    *tr = gegl_region_rectangle ((void *) &roi);
+          GeglRectangle  roi = rectangles[i];
+          GeglRegion    *tr = gegl_region_rectangle (&roi);
           gegl_region_subtract (processor->queued_region, tr);
           gegl_region_destroy (tr);
 
-          dr                          = g_malloc (sizeof (GeglRectangle));
-          *dr                         = roi;
-          processor->dirty_rectangles = g_slist_prepend (processor->dirty_rectangles, dr);
+          processor->dirty_rectangles = g_slist_prepend (processor->dirty_rectangles,
+                                                         g_slice_dup (GeglRectangle, &roi));
         }
       g_free (rectangles);
     }
@@ -543,7 +646,8 @@ gegl_processor_render (GeglProcessor *processor,
 
 
 /*#define ENABLE_THREADING*/
-#ifdef ENABLE_THREADING
+/*#ifdef ENABLE_THREADING*/
+#if 0
 
 gpointer render_thread (gpointer data)
 {
@@ -589,13 +693,22 @@ gegl_processor_work (GeglProcessor *processor,
     {
       return TRUE;
     }
+
+  if (GEGL_IS_OPERATION_SINK (processor->node->operation) &&
+      !gegl_operation_sink_needs_full (processor->node->operation))
+    {
+      if (progress)
+        *progress = 1.0;
+      return FALSE;
+    }
+
   cache = gegl_node_get_cache (processor->input);
 
-  if (processor->dynamic)
+  if (processor->context)
     {
       gegl_operation_process (processor->node->operation, cache, "foo");
-      gegl_node_remove_dynamic (processor->node, cache);
-      processor->dynamic = NULL;
+      gegl_node_remove_context (processor->node, cache);
+      processor->context = NULL;
       if (progress)
         *progress = 1.0;
       return TRUE;
@@ -622,11 +735,16 @@ gegl_processor_work (GeglProcessor *processor,
       return TRUE;
     }
 
-  if (processor->dynamic)
+  if (processor->context)
     {
-      gegl_operation_process (processor->node->operation, cache, "foo");
-      gegl_node_remove_dynamic (processor->node, cache);
-      processor->dynamic = NULL;
+      /* the actual writing to the destination */
+      gegl_operation_process (processor->node->operation,
+                              processor->context,
+                              "output"  /* ignored output_pad */,
+                              &processor->context->result_rect
+                              );
+      gegl_node_remove_context (processor->node, cache);
+      processor->context = NULL;
       if (progress)
         *progress = 1.0;
       return TRUE;
