@@ -16,15 +16,17 @@
  * Copyright 2006 Martin Nordholts <enselic@hotmail.com>
  */
 
-#include <glib.h>
-#include <glib/gprintf.h>
-#include <glib-object.h>
+#include "config.h"
+
 #include <string.h>
 
+#include <glib-object.h>
+
 #include "gegl-types.h"
 
 #include "gegl-color.h"
 
+
 enum
 {
   PROP_0,
@@ -256,7 +258,8 @@ gegl_color_get_rgba (GeglColor *self,
 {
   GeglColorPrivate *priv;
 
-  g_assert (self);
+  g_return_if_fail (GEGL_IS_COLOR (self));
+
   priv = GEGL_COLOR_GET_PRIVATE (self);
 
   *r = priv->rgba_color[0];
@@ -274,7 +277,8 @@ gegl_color_set_rgba (GeglColor *self,
 {
   GeglColorPrivate *priv;
 
-  g_assert (self);
+  g_return_if_fail (GEGL_IS_COLOR (self));
+
   priv = GEGL_COLOR_GET_PRIVATE (self);
 
   priv->rgba_color[0] = r;
@@ -342,26 +346,23 @@ gegl_color_set_from_string (GeglColor   *self,
 static gchar *
 gegl_color_get_string (GeglColor *color)
 {
-  GeglColorPrivate *priv;
-  gchar             buffer[512];
+  GeglColorPrivate *priv = GEGL_COLOR_GET_PRIVATE (color);
 
-  priv = GEGL_COLOR_GET_PRIVATE (color);
   if (priv->rgba_color[3] == 1.0)
     {
-      g_sprintf (buffer, "rgb(%1.4f, %1.4f, %1.4f)",
-                 priv->rgba_color[0],
-                 priv->rgba_color[1],
-                 priv->rgba_color[2]);
+      return g_strdup_printf ("rgb(%1.4f, %1.4f, %1.4f)",
+                              priv->rgba_color[0],
+                              priv->rgba_color[1],
+                              priv->rgba_color[2]);
     }
   else
     {
-      g_sprintf (buffer, "rgba(%1.4f, %1.4f, %1.4f, %1.4f)",
-                 priv->rgba_color[0],
-                 priv->rgba_color[1],
-                 priv->rgba_color[2],
-                 priv->rgba_color[3]);
+      return g_strdup_printf ("rgba(%1.4f, %1.4f, %1.4f, %1.4f)",
+                              priv->rgba_color[0],
+                              priv->rgba_color[1],
+                              priv->rgba_color[2],
+                              priv->rgba_color[3]);
     }
-  return g_strdup (buffer);
 }
 
 
@@ -414,6 +415,7 @@ gegl_color_new (const gchar *string)
 {
   if (string)
     return g_object_new (GEGL_TYPE_COLOR, "string", string, NULL);
+
   return g_object_new (GEGL_TYPE_COLOR, NULL);
 }
 
@@ -423,10 +425,8 @@ gegl_color_new (const gchar *string)
  * --------------------------------------------------------------------------
  */
 
-#define GEGL_TYPE_PARAM_COLOR             (gegl_param_color_get_type ())
-#define GEGL_PARAM_COLOR(obj)               (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEGL_TYPE_PARAM_COLOR, GeglParamColor))
-#define GEGL_IS_PARAM_COLOR(obj)            (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEGL_TYPE_PARAM_COLOR))
-#define GEGL_IS_PARAM_COLOR_CLASS(klass)    (G_TYPE_CHECK_CLASS_TYPE ((klass), GEGL_TYPE_PARAM_COLOR))
+#define GEGL_PARAM_COLOR(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEGL_TYPE_PARAM_COLOR, GeglParamColor))
+#define GEGL_IS_PARAM_COLOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEGL_TYPE_PARAM_COLOR))
 
 typedef struct _GeglParamColor GeglParamColor;
 
@@ -464,10 +464,6 @@ value_set_default (GParamSpec *param_spec,
 {
   GeglParamColor *gegl_color = GEGL_PARAM_COLOR (param_spec);
 
-  g_object_ref (gegl_color->default_color); /* XXX:
-                                               not sure why this is needed,
-                                               but a reference is leaked
-                                               unless it his here */
   g_value_set_object (value, gegl_color->default_color);
 }
 
@@ -510,6 +506,8 @@ gegl_param_spec_color (const gchar *name,
                                        name, nick, blurb, flags);
 
   param_color->default_color = default_color;
+  if (default_color)
+    g_object_ref (default_color);
 
   return G_PARAM_SPEC (param_color);
 }
