@@ -23,9 +23,7 @@
 
 G_BEGIN_DECLS
 
-#ifndef GEGL_TYPE_COLOR
 #define GEGL_TYPE_COLOR            (gegl_color_get_type ())
-#endif
 #define GEGL_COLOR(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEGL_TYPE_COLOR, GeglColor))
 #define GEGL_COLOR_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass),  GEGL_TYPE_COLOR, GeglColorClass))
 #define GEGL_IS_COLOR(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEGL_TYPE_COLOR))
@@ -36,29 +34,35 @@ typedef struct _GeglColorClass  GeglColorClass;
 
 struct _GeglColor
 {
-  GObject parent;
+  GObject parent_instance;
 };
 
 struct _GeglColorClass
 {
-  GObjectClass parent;
+  GObjectClass parent_class;
 };
 
-void         gegl_color_get_rgba               (GeglColor    *self,
-                                                gfloat       *r,
-                                                gfloat       *g,
-                                                gfloat       *b,
-                                                gfloat       *a);
+GType        gegl_color_get_type               (void) G_GNUC_CONST;
 
-void         gegl_color_set_rgba               (GeglColor    *self,
+GeglColor *  gegl_color_new                    (const gchar *string);
+
+void         gegl_color_get_rgba               (GeglColor   *self,
+                                                gfloat      *r,
+                                                gfloat      *g,
+                                                gfloat      *b,
+                                                gfloat      *a);
+
+void         gegl_color_set_rgba               (GeglColor   *self,
                                                 gfloat       r,
                                                 gfloat       g,
                                                 gfloat       b,
                                                 gfloat       a);
 
-GeglColor *  gegl_color_new                    (const gchar *string);
 
-GType        gegl_color_get_type               (void) G_GNUC_CONST;
+#define GEGL_TYPE_PARAM_COLOR           (gegl_param_color_get_type ())
+#define GEGL_IS_PARAM_SPEC_COLOR(pspec) (G_TYPE_CHECK_INSTANCE_TYPE ((pspec), GEGL_TYPE_PARAM_COLOR))
+
+GType        gegl_param_color_get_type         (void) G_GNUC_CONST;
 
 GParamSpec * gegl_param_spec_color             (const gchar *name,
                                                 const gchar *nick,
@@ -72,8 +76,6 @@ GParamSpec * gegl_param_spec_color_from_string (const gchar *name,
                                                 const gchar *default_color_string,
                                                 GParamFlags  flags);
 
-GType        gegl_param_color_get_type         (void) G_GNUC_CONST;
-
 G_END_DECLS
 
 #endif /* __GEGL_COLOR_H__ */
