@@ -23,15 +23,16 @@
 /* TODO: make rect calculations depend on the sampling kernel of the
  *       interpolation filter used */
 
-#include <string.h>
 #include <math.h>
 #include <gegl-plugin.h>
-#include <gegl-module.h>
+#include <graph/gegl-pad.h>
+#include <graph/gegl-node.h>
+#include <graph/gegl-connection.h>
+/*#include "buffer/gegl-sampler.h"*/
 
 #include "affine.h"
 #include "module.h"
 #include "matrix.h"
-#include "buffer/gegl-sampler.h"
 
 enum
 {
@@ -44,34 +45,36 @@ enum
 
 /* *** static prototypes *** */
 
-static void          get_property         (GObject       *object,
-                                           guint          prop_id,
-                                           GValue        *value,
-                                           GParamSpec    *pspec);
-static void          set_property         (GObject       *object,
-                                           guint          prop_id,
-                                           const GValue  *value,
-                                           GParamSpec    *pspec);
-static void          bounding_box         (gdouble       *points,
-                                           gint           num_points,
-                                           GeglRectangle *output);
-static gboolean      is_intermediate_node (OpAffine      *affine);
-static gboolean      is_composite_node    (OpAffine      *affine);
-static void          get_source_matrix    (OpAffine      *affine,
-                                           Matrix3        output);
-static GeglRectangle get_defined_region   (GeglOperation *op);
-static GeglRectangle compute_affected_region  (GeglOperation *operation,
-                                           const gchar   *input_pad,
-                                           GeglRectangle  region);
-static GeglRectangle compute_input_request (GeglOperation *self,
-                                           const gchar   *input_pad,
-                                           GeglRectangle *region);
+static void          get_property            (GObject             *object,
+                                              guint                prop_id,
+                                              GValue              *value,
+                                              GParamSpec          *pspec);
+static void          set_property            (GObject             *object,
+                                              guint                prop_id,
+                                              const GValue        *value,
+                                              GParamSpec          *pspec);
+static void          bounding_box            (gdouble             *points,
+                                              gint                 num_points,
+                                              GeglRectangle       *output);
+static gboolean      is_intermediate_node    (OpAffine            *affine);
+static gboolean      is_composite_node       (OpAffine            *affine);
+static void          get_source_matrix       (OpAffine            *affine,
+                                              Matrix3              output);
+static GeglRectangle get_bounding_box       (GeglOperation       *op);
+static GeglRectangle get_invalidated_by_change (GeglOperation       *operation,
+                                              const gchar         *input_pad,
+                                              const GeglRectangle *input_region);
+static GeglRectangle get_required_for_output(GeglOperation       *self,
+                                              const gchar         *input_pad,
+                                              const GeglRectangle *region);
 
-static gboolean      process              (GeglOperation *op,
-                                           gpointer       context_id);
-static GeglNode    * detect               (GeglOperation *operation,
-                                           gint           x,
-                                           gint           y);
+static gboolean      process                 (GeglOperation       *op,
+                                              GeglBuffer          *input,
+                                              GeglBuffer          *output,
+                                              const GeglRectangle *result);
+static GeglNode    * detect                  (GeglOperation       *operation,
+                                              gint                 x,
+                                              gint                 y);
 
 /* ************************* */
 
@@ -118,8 +121,7 @@ op_affine_get_type (void)
 /* ************************* */
 
 static void
-prepare (GeglOperation *operation,
-         gpointer       context_id)
+prepare (GeglOperation *operation)
 {
   Babl *format = babl_format ("RaGaBaA float");
 
@@ -138,9 +140,9 @@ op_affine_class_init (OpAffineClass *klass)
   gobject_class->set_property = set_property;
   gobject_class->get_property = get_property;
 
-  op_class->compute_affected_region = compute_affected_region;
-  op_class->get_defined_region  = get_defined_region;
-  op_class->compute_input_request = compute_input_request;
+  op_class->get_invalidated_by_change = get_invalidated_by_change;
+  op_class->get_bounding_box = get_bounding_box;
+  op_class->get_required_for_output = get_required_for_output;
   op_class->detect = detect;
   op_class->categories = "transform";
   op_class->prepare = prepare;
@@ -205,17 +207,23 @@ get_property (GObject    *object,
   switch (prop_id)
     {
     case PROP_ORIGIN_X:
-      g_value_set_double (value, self->origin_x); break;
+      g_value_set_double (value, self->origin_x);
+      break;
     case PROP_ORIGIN_Y:
-      g_value_set_double (value, self->origin_y); break;
+      g_value_set_double (value, self->origin_y);
+      break;
     case PROP_FILTER:
-      g_value_set_string (value, self->filter); break;
+      g_value_set_string (value, self->filter);
+      break;
     case PROP_HARD_EDGES:
-      g_value_set_boolean (value, self->hard_edges); break;
+      g_value_set_boolean (value, self->hard_edges);
+      break;
     case PROP_LANCZOS_WIDTH:
-      g_value_set_int (value, self->lanczos_width); break;
+      g_value_set_int (value, self->lanczos_width);
+      break;
     default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec); break;
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
     }
 }
 
@@ -230,18 +238,24 @@ set_property (GObject      *object,
   switch (prop_id)
     {
     case PROP_ORIGIN_X:
-      self->origin_x = g_value_get_double (value); break;
+      self->origin_x = g_value_get_double (value);
+      break;
     case PROP_ORIGIN_Y:
-      self->origin_y = g_value_get_double (value); break;
+      self->origin_y = g_value_get_double (value);
+      break;
     case PROP_FILTER:
       g_free (self->filter);
-      self->filter = g_strdup (g_value_get_string (value)); break;
+      self->filter = g_strdup (g_value_get_string (value));
+      break;
     case PROP_HARD_EDGES:
-      self->hard_edges = g_value_get_boolean (value); break;
+      self->hard_edges = g_value_get_boolean (value);
+      break;
     case PROP_LANCZOS_WIDTH:
-      self->lanczos_width = g_value_get_int (value); break;
+      self->lanczos_width = g_value_get_int (value);
+      break;
     default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec); break;
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
     }
 }
 
@@ -288,8 +302,7 @@ static gboolean
 is_intermediate_node (OpAffine *affine)
 {
   GSList        *connections;
-  GeglOperation *op = GEGL_OPERATION (affine),
-                *sink;
+  GeglOperation *op = GEGL_OPERATION (affine);
 
   connections = gegl_pad_get_connections (gegl_node_get_pad (op->node,
                                                              "output"));
@@ -298,9 +311,12 @@ is_intermediate_node (OpAffine *affine)
 
   do
     {
+      GeglOperation *sink;
+
       sink = gegl_connection_get_sink_node (connections->data)->operation;
+
       if (! IS_OP_AFFINE (sink) ||
-          strcasecmp (affine->filter, OP_AFFINE (sink)->filter))
+          strcmp (affine->filter, OP_AFFINE (sink)->filter))
         return FALSE;
     }
   while ((connections = g_slist_next (connections)));
@@ -312,8 +328,8 @@ static gboolean
 is_composite_node (OpAffine *affine)
 {
   GSList        *connections;
-  GeglOperation *op = GEGL_OPERATION (affine),
-                *source;
+  GeglOperation *op = GEGL_OPERATION (affine);
+  GeglOperation *source;
 
   connections = gegl_pad_get_connections (gegl_node_get_pad (op->node,
                                                              "input"));
@@ -323,7 +339,7 @@ is_composite_node (OpAffine *affine)
   source = gegl_connection_get_source_node (connections->data)->operation;
 
   return (IS_OP_AFFINE (source) &&
-          ! strcasecmp (affine->filter, OP_AFFINE (source)->filter));
+          ! strcmp (affine->filter, OP_AFFINE (source)->filter));
 }
 
 static void
@@ -331,8 +347,8 @@ get_source_matrix (OpAffine *affine,
                    Matrix3   output)
 {
   GSList        *connections;
-  GeglOperation *op = GEGL_OPERATION (affine),
-                *source;
+  GeglOperation *op = GEGL_OPERATION (affine);
+  GeglOperation *source;
 
   connections = gegl_pad_get_connections (gegl_node_get_pad (op->node,
                                                              "input"));
@@ -345,7 +361,7 @@ get_source_matrix (OpAffine *affine,
 }
 
 static GeglRectangle
-get_defined_region (GeglOperation *op)
+get_bounding_box (GeglOperation *op)
 {
   OpAffine      *affine  = (OpAffine *) op;
   OpAffineClass *klass   = OP_AFFINE_GET_CLASS (affine);
@@ -354,8 +370,8 @@ get_defined_region (GeglOperation *op)
   gdouble        have_points [8];
   gint           i;
 
-  if (gegl_operation_source_get_defined_region (op, "input"))
-  in_rect = *gegl_operation_source_get_defined_region (op, "input");
+  if (gegl_operation_source_get_bounding_box (op, "input"))
+  in_rect = *gegl_operation_source_get_bounding_box (op, "input");
 
   /* invoke child's matrix creation function */
   g_assert (klass->create_matrix);
@@ -378,7 +394,7 @@ get_defined_region (GeglOperation *op)
       return in_rect;
     }
 
-  if (! strcasecmp (affine->filter, "linear"))
+  if (! strcmp (affine->filter, "linear"))
     {
       if (affine->hard_edges)
         {
@@ -422,10 +438,10 @@ detect (GeglOperation *operation,
 {
   GeglNode *source_node = gegl_operation_get_source_node (operation, "input");
 
-  OpAffine      *affine = (OpAffine *) operation;
-  Matrix3        inverse;
-  gdouble        need_points [2];
-  gint           i;
+  OpAffine *affine = (OpAffine *) operation;
+  Matrix3   inverse;
+  gdouble   need_points [2];
+  gint      i;
 
   if (is_intermediate_node (affine) ||
       matrix3_is_identity (inverse))
@@ -443,12 +459,14 @@ detect (GeglOperation *operation,
     matrix3_transform_point (inverse,
                              need_points + i, need_points + i + 1);
 
-  return gegl_operation_detect (source_node->operation, need_points[0], need_points[1]);
+  return gegl_operation_detect (source_node->operation,
+                                need_points[0], need_points[1]);
 }
 
-static GeglRectangle compute_input_request (GeglOperation *op,
-                                            const gchar   *input_pad,
-                                            GeglRectangle *region)
+static GeglRectangle
+get_required_for_output (GeglOperation       *op,
+                         const gchar         *input_pad,
+                         const GeglRectangle *region)
 {
   OpAffine      *affine = (OpAffine *) op;
   Matrix3        inverse;
@@ -488,7 +506,7 @@ static GeglRectangle compute_input_request (GeglOperation *op,
                              need_points + i, need_points + i + 1);
   bounding_box (need_points, 4, &need_rect);
 
-  if (! strcasecmp (affine->filter, "linear"))
+  if (! strcmp (affine->filter, "linear"))
     {
       if (affine->hard_edges)
         {
@@ -508,15 +526,16 @@ static GeglRectangle compute_input_request (GeglOperation *op,
 }
 
 static GeglRectangle
-compute_affected_region (GeglOperation *op,
-                     const gchar   *input_pad,
-                     GeglRectangle  region)
+get_invalidated_by_change (GeglOperation       *op,
+                           const gchar         *input_pad,
+                           const GeglRectangle *input_region)
 {
   OpAffine      *affine  = (OpAffine *) op;
   OpAffineClass *klass   = OP_AFFINE_GET_CLASS (affine);
   GeglRectangle  affected_rect;
   gdouble        affected_points [8];
   gint           i;
+  GeglRectangle  region = *input_region;
 
   /* invoke child's matrix creation function */
   g_assert (klass->create_matrix);
@@ -539,7 +558,7 @@ compute_affected_region (GeglOperation *op,
       return region;
     }
 
-  if (! strcasecmp (affine->filter, "linear"))
+  if (! strcmp (affine->filter, "linear"))
     {
       if (affine->hard_edges)
         {
@@ -577,7 +596,12 @@ compute_affected_region (GeglOperation *op,
 
 }
 
-void
+void  gegl_sampler_prepare     (GeglSampler *self);
+  /*XXX: Eeeek, obsessive avoidance of public headers, the API needed to
+   *     satisfy this use case should probably be provided.
+   */
+
+static void
 affine_generic (GeglBuffer        *dest,
                 GeglBuffer        *src,
                 Matrix3            matrix,
@@ -593,8 +617,8 @@ affine_generic (GeglBuffer        *dest,
                         u_float,
                         v_float;
 
-  Babl                 *format; 
-  
+  Babl                 *format;
+
   gint                  dest_pixels;
 
   format = babl_format ("RaGaBaA float");
@@ -649,25 +673,19 @@ affine_generic (GeglBuffer        *dest,
       v_start += inverse [1][1];
     }
   gegl_buffer_sample_cleanup (src);
-  gegl_buffer_set (dest, NULL, format, dest_buf);
+  gegl_buffer_set (dest, NULL, format, dest_buf, GEGL_AUTO_ROWSTRIDE);
   g_free (dest_buf);
 }
 
 
 
 static gboolean
-process (GeglOperation *operation,
-         gpointer       context_id)
+process (GeglOperation       *operation,
+         GeglBuffer          *input,
+         GeglBuffer          *output,
+         const GeglRectangle *result)
 {
-  OpAffine      *affine = (OpAffine *) operation;
-  GeglBuffer    *input;
-  GeglBuffer    *output;
-  GeglRectangle *result;
- 
-  result = gegl_operation_result_rect (operation, context_id);
-
-  input = gegl_operation_get_source (operation, context_id, "input");
-  output = gegl_operation_get_target (operation, context_id, "output");
+  OpAffine            *affine = (OpAffine *) operation;
 
   /*g_warning ("%i,%i %ix%i | %i,%i %ix%i | %i,%i  %ix%i",
      input->x, input->y, input->width, input->height,
@@ -686,7 +704,7 @@ process (GeglOperation *operation,
                 but should still be faster than a full resampling when it
                 is not needed */
   else if (matrix3_is_translate (affine->matrix) &&
-           (! strcasecmp (affine->filter, "nearest") ||
+           (! strcmp (affine->filter, "nearest") ||
             (affine->matrix [0][2] == (gint) affine->matrix [0][2] &&
              affine->matrix [1][2] == (gint) affine->matrix [1][2])))
     {
@@ -708,9 +726,9 @@ process (GeglOperation *operation,
   else
     {
       /* XXX: add back more samplers */
-      affine_generic (output, input, affine->matrix, gegl_buffer_interpolation_from_string (
-         affine->filter));
+      affine_generic (output, input, affine->matrix,
+                      gegl_buffer_interpolation_from_string (affine->filter));
     }
-  g_object_unref (input);
+
   return TRUE;
 }
