@@ -15,63 +15,19 @@
  *
  * Copyright 2006 Øyvind Kolås <pippin@gimp.org>
  */
-#if GEGL_CHANT_PROPERTIES
- 
-gegl_chant_double (radius, 0.0, 50.0, 4.0,
-  "Radius of square pixel region, (width and height will be radius*2+1.")
+#ifdef GEGL_CHANT_PROPERTIES
+
+gegl_chant_double (radius, "Radius", 0.0, 50.0, 4.0,
+  "Radius of square pixel region (width and height will be radius*2+1)")
 
 #else
 
-#define GEGL_CHANT_NAME            kuwahara_min
-#define GEGL_CHANT_SELF            "kuwahara-min.c"
-#define GEGL_CHANT_DESCRIPTION     "Edge preserving min filter"
-#define GEGL_CHANT_CATEGORIES      "misc"
-
-#define GEGL_CHANT_AREA_FILTER
+#define GEGL_CHANT_TYPE_AREA_FILTER
+#define GEGL_CHANT_C_FILE       "kuwahara-min.c"
 
 #include "gegl-chant.h"
 #include <math.h>
 
-static void
-kuwahara (GeglBuffer *src,
-          GeglBuffer *dst,
-          gint        radius);
-
-static gboolean
-process (GeglOperation *operation,
-         gpointer       context_id)
-{
-  GeglOperationFilter *filter;
-  GeglChantOperation  *self;
-  GeglBuffer          *input;
-  GeglBuffer          *output;
-
-  filter = GEGL_OPERATION_FILTER (operation);
-  self   = GEGL_CHANT_OPERATION (operation);
-
-
-  input = GEGL_BUFFER (gegl_operation_get_data (operation, context_id, "input"));
-    {
-      GeglRectangle   *result = gegl_operation_result_rect (operation, context_id);
-      GeglBuffer      *temp_in;
-      GeglRectangle    compute  = gegl_operation_compute_input_request (operation, "inputt", gegl_operation_need_rect (operation, context_id));
-
-      temp_in = gegl_buffer_create_sub_buffer (input, &compute);
-      output = gegl_buffer_new (&compute, babl_format ("RGBA float"));
-
-      kuwahara (temp_in, output, self->radius);
-      g_object_unref (temp_in);
-      
-
-      {
-        GeglBuffer *cropped = gegl_buffer_create_sub_buffer (output, result);
-        gegl_operation_set_data (operation, context_id, "output", G_OBJECT (cropped));
-        g_object_unref (output);
-      }
-    }
-  return  TRUE;
-}
-
 static inline void
 compute_rectangle (gfloat *buf,
                    gint    buf_width,
@@ -90,7 +46,7 @@ compute_rectangle (gfloat *buf,
   gfloat  max   = -1000000000.0;
   gfloat  min   =  1000000000.0;
   gfloat  mean  =  0.0;
-  gint    count =  0;
+  glong   count =  0;
 
   gint offset = (y0 * buf_width + x0) * 4 + component;
 
@@ -132,8 +88,8 @@ kuwahara (GeglBuffer *src,
   gfloat *src_buf;
   gfloat *dst_buf;
 
-  src_buf = g_malloc0 (gegl_buffer_get_pixel_count (src) * 4 * 4);
-  dst_buf = g_malloc0 (gegl_buffer_get_pixel_count (dst) * 4 * 4);
+  src_buf = g_new0 (gfloat, gegl_buffer_get_pixel_count (src) * 4);
+  dst_buf = g_new0 (gfloat, gegl_buffer_get_pixel_count (dst) * 4);
 
   gegl_buffer_get (src, 1.0, NULL, babl_format ("RGBA float"), src_buf, GEGL_AUTO_ROWSTRIDE);
 
@@ -161,7 +117,7 @@ kuwahara (GeglBuffer *src,
                                component,
                                &min, /* min */
                                NULL, /* max */
-                               NULL, 
+                               NULL,
                                &variance);
             if (variance<best)
               {
@@ -179,7 +135,7 @@ kuwahara (GeglBuffer *src,
                                component,
                                &min, /* min */
                                NULL, /* max */
-                               NULL, 
+                               NULL,
                                &variance);
             if (variance<best)
               {
@@ -197,7 +153,7 @@ kuwahara (GeglBuffer *src,
                                component,
                                &min, /* min */
                                NULL, /* max */
-                               NULL, 
+                               NULL,
                                &variance);
             if (variance<best)
               {
@@ -215,8 +171,9 @@ kuwahara (GeglBuffer *src,
                                component,
                                &min, /* min */
                                NULL, /* max */
-                               NULL, 
+                               NULL,
                                &variance);
+
             if (variance<best)
               {
                 value = min;
@@ -228,17 +185,54 @@ kuwahara (GeglBuffer *src,
           offset++;
       }
 
-  gegl_buffer_set (dst, NULL, babl_format ("RGBA float"), dst_buf);
+  gegl_buffer_set (dst, NULL, babl_format ("RGBA float"), dst_buf, GEGL_AUTO_ROWSTRIDE);
   g_free (src_buf);
   g_free (dst_buf);
 }
 
-static void tickle (GeglOperation *operation)
+static void prepare (GeglOperation *operation)
 {
   GeglOperationAreaFilter *area = GEGL_OPERATION_AREA_FILTER (operation);
-  GeglChantOperation      *filter = GEGL_CHANT_OPERATION (operation);
+
   area->left = area->right = area->top = area->bottom =
-  ceil (filter->radius);
+      ceil (GEGL_CHANT_PROPERTIES (operation)->radius);
+  gegl_operation_set_format (operation, "output", babl_format ("RGBA float"));
+}
+
+static gboolean
+process (GeglOperation       *operation,
+         GeglBuffer          *input,
+         GeglBuffer          *output,
+         const GeglRectangle *result)
+{
+  GeglChantO   *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglBuffer   *temp_in;
+  GeglRectangle compute = gegl_operation_get_required_for_output (operation, "input", result);
+
+  temp_in = gegl_buffer_create_sub_buffer (input, &compute);
+
+  kuwahara (temp_in, output, o->radius);
+  g_object_unref (temp_in);
+
+  return  TRUE;
+}
+
+
+static void
+gegl_chant_class_init (GeglChantClass *klass)
+{
+  GeglOperationClass       *operation_class;
+  GeglOperationFilterClass *filter_class;
+
+  operation_class = GEGL_OPERATION_CLASS (klass);
+  filter_class    = GEGL_OPERATION_FILTER_CLASS (klass);
+
+  filter_class->process = process;
+  operation_class->prepare = prepare;
+
+  operation_class->name        = "kuwahara-min";
+  operation_class->categories  = "misc";
+  operation_class->description = "Edge preserving min filter";
 }
 
 #endif
